/************************************************************************
 *																		*
 *	Copyright (c) 2000 by			David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"
#include "panel.h"

	PanelManager :: PanelManager()
{
	m_pParent = NULL;
	m_pPanels = NULL;
	m_pBackBtn = NULL;
	m_pNextBtn = NULL;
	m_pFinishBtn = NULL;
	m_nPanelCount = 0;
}

	PanelManager ::	~PanelManager()
{
	PANEL **	pPanels = m_pPanels;
	int nIndex;

	if (pPanels != NULL)
	{
		for (nIndex = 0; pPanels[nIndex] != NULL; ++nIndex)
			delete pPanels[nIndex];
		delete [] m_pPanels;
	}
}


/*----------------------------------------------------------------------+
 |																		|
 |	description:														|
 |		Create -- initialize a panel manager object and supply it with	|
 |		its owner's CWnd * (needed so the panel manager can call CWnd	|
 |		member functions) and pointers to the BACK, NEXT and FINISH		|
 |		buttons (which the parent dialog must have to support the		|
 |		multi-panel dialog concept).									|
 |	args:																|
 |		CDialog * pOwner	:	the "this" of the owner (which must be 	|
 |								an object derived from CDialog)			|
 |		CWnd *	  pBackBtn	:	the CButton * for the BACK button (It	|
 |								can be NULL, in which case pNextBtn		|
 |								must also be NULL.  This is the type of	|
 |								mulit-panel dialog which displays only	|
 |								a single panel but that panel is picked	|
 |								as the dialog is initialized.)			|
 |		CWnd *	  pNextBtn	:	the CButton * for the NEXT button  (It	|
 |								cannot be NULL unless pBackBtn is.)		|
 |		CWnd *	  pFinishBtn:	the CButton * for the FINISH button (it	|
 |								can be NULL, indicating that the dialog	|
 |								buttons are just BACK, NEXT and CLOSE)	|
 |																		|
 |	returns:															|
 |		TRUE	if success												|
 |		FALSE	if any pointer is NULL (except for a pXXXXBtn, which is	|
 |				allowed to be NULL) or if pOwner isn't a CDialog or	if	|
 |				any pXxxxBtn (except pXXXXBtn if NULL) has no HWnd		|
 |																		|
 +----------------------------------------------------------------------*/

BOOL	PanelManager :: Create(CDialog * pOwner,
								CWnd * pBackBtn, CWnd * pNextBtn,
								CWnd * pFinishBtn)
{
	if ((pOwner == NULL) || 
		((pBackBtn == NULL) && (pNextBtn != NULL)) ||
		((pBackBtn != NULL) && (pNextBtn == NULL)))
	{
		AfxMessageBox("PanelManager::Create -- got unexpected NULL pointer argument.");
		return FALSE;
	}
	if ((pOwner -> m_hWnd == NULL) || 
		((pBackBtn != NULL) && (pBackBtn -> m_hWnd == NULL)) ||
		((pNextBtn != NULL) && (pNextBtn -> m_hWnd == NULL)) ||
		((pFinishBtn != NULL) && (pFinishBtn -> m_hWnd == NULL)))
	{
		AfxMessageBox("PanelManager::Create -- got bad m_hWnd from argument.");
		return FALSE;
	}
	if (!(pOwner -> IsKindOf(RUNTIME_CLASS(CDialog))))
	{
		AfxMessageBox("PanelManager::Create -- owner is not a CDialog object.");
		return FALSE;
	}

//	Argument list checks out as valid.

	m_pParent = pOwner;
	m_pBackBtn = pBackBtn;
	m_pNextBtn = pNextBtn;
	m_pFinishBtn = pFinishBtn;
	return TRUE;
}


/*----------------------------------------------------------------------+
 |																		|
 |	description:														|
 |		AddPanel -- adds a new panel to the array of panels maintained	|
 |					by the panel manager object							|
 |	args:																|
 |		int		nXOffset	:	the adjustment to the position of the	|
 |								left edge of each control in the panel	|
 |								which must be made when positioning the |
 |								panel.									|
 |		int		nYOffset	:	the adjustment to the position of the	|
 |								top edge of each control in the panel	|
 |								which must be made when positioning the |
 |								panel.									|
 |		int		nControlCount:	the number of controls on the panel		|
 |		UINT *	pControls	:	a pointer to the list of control IDs	|
 |		UINT	nFocusIndex:	the index of the control ID which is to	|
 |								get focus when we do a JumpToPanel.  If	|
 |								this value is -1, no control gets focus	|
 |																		|
 |	returns:															|
 |		TRUE	if success												|
 |		FALSE	if nControlCount is less than one, if pControls is		|
 |				NULL, if m_pParent is NULL (we never called the Panel	|
 |				Manager's Create method), if the value of nFocusIndex	|
 |				is greater than nControlCount, if any control ID in		|
 |				the list is not owned by m_pParent or if the new		|
 |				operator fails while allocating the pointer array or	|
 |				the panel object.										|
 |																		|
 +----------------------------------------------------------------------*/

BOOL	PanelManager::AddPanel(int nXOffset, int nYOffset,
								int nControlCount, UINT * pControls,
								int nFocusIndex)
{
	PANEL **	pPanels;
	int			nIndex;
	CString		strTemp;

	if (m_pParent == NULL)
	{
		AfxMessageBox("PanelManager::AddPanel -- must call Create method "
						"successfully first!");
		return FALSE;
	}
	if (nControlCount < 1)
	{
		AfxMessageBox("PanelManager::AddPanel -- invalid control count.");
		return FALSE;
	}
	if (nFocusIndex >= nControlCount)
	{
		AfxMessageBox("PanelManager::AddPanel -- invalid focus index.");
		return FALSE;
	}
	if (pControls == NULL)
	{
		AfxMessageBox("PanelManager::AddPanel -- NULL controls pointer.");
		return FALSE;
	}
	for (nIndex = 0; nIndex < nControlCount; ++nIndex)
	{
		if (m_pParent -> GetDlgItem(pControls[nIndex]) == NULL)
		{
			strTemp.Format("%d (index = %d).",
				pControls[nIndex], nIndex);
			AfxMessageBox("PanelManager::AddPanel -- invalid control ID " +
							strTemp);
			return FALSE;
		}
	}
	pPanels = (PANEL **) new (PANEL *[++m_nPanelCount + 1]);
	if (pPanels == NULL)
	{
		AfxMessageBox("PanelManager::AddPanel -- alloc failure.");
		return FALSE;
	}
	for (nIndex = 0; nIndex < m_nPanelCount - 1; ++nIndex)
		pPanels[nIndex] = m_pPanels[nIndex];
	if ((pPanels[nIndex] = (PANEL *) new PANEL) == NULL)
	{
		AfxMessageBox("PanelManager::AddPanel -- alloc failure.");
		return FALSE;
	}
	pPanels[nIndex] -> iXOffset = nXOffset;
	pPanels[nIndex] -> iYOffset = nYOffset;
	pPanels[nIndex] -> nControlCount = nControlCount;
	pPanels[nIndex] -> pControls = pControls;
	pPanels[nIndex] -> nFocusControl = nFocusIndex;
	++nIndex;
	pPanels[nIndex] = NULL;
	delete [] m_pPanels;
	m_pPanels = pPanels;
	return TRUE;
}

void	PanelManager::RepositionPanels()
{
	PANEL **	pP = m_pPanels;
	int			nIndex;

	for (nIndex = 0; pP[nIndex] != NULL; ++nIndex)
	{
		PositionPanel(*pP[nIndex]);
	}
}

void	PanelManager::PositionPanel(struct PANEL P)
{
	int		nIndex;
	CWnd *	pWnd;

	for (nIndex = 0; nIndex < P.nControlCount; ++nIndex)
	{
		pWnd = m_pParent -> GetDlgItem(P.pControls[nIndex]);
		PositionControl(pWnd, P.iXOffset, P.iYOffset);
	}
}


/*----------------------------------------------------------------------+
 |																		|
 |	description:														|
 |		PositionControl -- changes the position of a given control		|
 |	args:																|
 |		CWnd *	pWnd		:	pointer to the control.					|
 |		int		nXOffset	:	the adjustment to the position of the	|
 |								left edge of the control.				|
 |		int		nYOffset	:	the adjustment to the position of the	|
 |								top edge of the control.				|
 |																		|
 |	returns:															|
 |		TRUE	if success												|
 |		FALSE	pWnd is NULL or pWnd -> m_hWnd is invalid.				|
 |																		|
 +----------------------------------------------------------------------*/

BOOL	PanelManager::PositionControl(CWnd * pWnd, int nX, int nY)
{
	CRect	rWnd;
	CRect	rPosition(nX, nY, 0, 0);

	if (pWnd == NULL)
	{
		AfxMessageBox("PanelManager::PositionControl -- "
						"NULL pointer argument.");
		return FALSE;
	}
	if (pWnd -> m_hWnd == NULL)
	{
		AfxMessageBox("PanelManager::PositionControl -- "
						"invalid hWnd of argument.");
		return FALSE;
	}
	m_pParent -> MapDialogRect(&rPosition);
	pWnd -> GetWindowRect(rWnd);
	m_pParent -> ScreenToClient(rWnd);
	rWnd.left += rPosition.left;
	rWnd.right += rPosition.left;
	rWnd.top += rPosition.top;
	rWnd.bottom += rPosition.top;
	pWnd -> MoveWindow(rWnd);
	return TRUE;
}


void	PanelManager::ResizeWindow(int nWidth, int nHeight)
{
	CRect	rSize(0, 0, nWidth + 2, nHeight + 12);	//TODO: figure out why the width & height need adjusting.
	CRect	rWnd;

	m_pParent -> MapDialogRect(&rSize);
	m_pParent -> GetWindowRect(&rWnd);
	rWnd.right = rWnd.left + rSize.right;
	rWnd.bottom = rWnd.top + rSize.bottom;
	m_pParent -> MoveWindow(&rWnd);
}


void	PanelManager::HidePanel(struct PANEL P)
{
	DisablePanel(P.nControlCount, P.pControls);
}


void	PanelManager::ShowPanel(struct PANEL P)
{
	int		nIndex;
	CWnd *	pWnd;
	for (nIndex = 0; nIndex < P.nControlCount; ++nIndex)
	{
		pWnd = m_pParent -> GetDlgItem(P.pControls[nIndex]);
		pWnd -> ShowWindow(SW_NORMAL);
	}
}


void	PanelManager :: JumpToPanel(struct PANEL P)
{
	int		nIndex = P.nFocusControl;
	if (nIndex >= 0)
	{
		CWnd *	pWnd = m_pParent -> GetDlgItem(P.pControls[nIndex]);
		m_pParent -> GotoDlgCtrl(pWnd);
	}
}


BOOL	PanelManager :: Paginate(int nPage)
{
	PANEL **	pP = m_pPanels;
	int			nIndex;
	int			nActive = 0;

	if ((nPage < 0) || (nPage >= m_nPanelCount))
	{
		AfxMessageBox("PanelManager::Paginate -- "
						"invalid panel number argument.");
		return FALSE;
	}
	for (nIndex = 0; nIndex < m_nPanelCount; ++ nIndex)
	{
		if (nIndex == nPage)
		{
			ShowPanel(*pP[nIndex]);
			nActive = nIndex;
		}
		else HidePanel(*pP[nIndex]);
	}
	if (nPage == 0)
	{

//	First page -- hide BACK if we have one.

		if (m_pBackBtn != NULL)
		{
			m_pBackBtn -> ShowWindow(SW_HIDE);
		}
	}
	else 
	{

//	Not first page.  Show BACK if we have one.

		if (m_pBackBtn != NULL)
		{
			m_pBackBtn -> ShowWindow(SW_NORMAL);
		}
	}
	if (nPage == (m_nPanelCount - 1))
	{

//	Last page -- display FINISH if we have one.  Hide NEXT if we have one.
//	If we don't have FINISH, set focus to BACK, else set focus to FINISH.
//	(The above tests ensure that BACK is shown.)

		if (m_pNextBtn != NULL)
		{
			m_pNextBtn -> ShowWindow(SW_HIDE);
		}
		if (m_pFinishBtn != NULL)
		{
			m_pFinishBtn -> ShowWindow(SW_NORMAL);
			m_pParent -> GotoDlgCtrl(m_pFinishBtn);
		}
		else
		{
			if (m_pBackBtn != NULL)
			{
				m_pParent -> GotoDlgCtrl(m_pBackBtn);
			}
		}
	}
	else
	{

//	Not last page.  If we have both FINISH and NEXT, show NEXT
//	and hide FINISH.  Else if we have only NEXT, show NEXT.  Else
//	if we have only FINISH, show FINISH.  Set focus to whatever we
//	show.

		if ((m_pFinishBtn != NULL) && (m_pNextBtn != NULL))
		{
			m_pFinishBtn -> ShowWindow(SW_HIDE);
			m_pNextBtn -> ShowWindow(SW_NORMAL);
			m_pParent -> GotoDlgCtrl(m_pNextBtn);
		}
		else if (m_pNextBtn != NULL)
		{
			m_pNextBtn -> ShowWindow(SW_NORMAL);
			m_pParent -> GotoDlgCtrl(m_pNextBtn);
		}
		else if (m_pFinishBtn != NULL)
		{
			m_pFinishBtn -> ShowWindow(SW_NORMAL);
			m_pParent -> GotoDlgCtrl(m_pFinishBtn);
		}
	}
	JumpToPanel(*pP[nActive]);
	return TRUE;
}


BOOL	PanelManager :: DisablePanel(int nControlCount, UINT * pControls)
{
	int		nIndex;
	CWnd *	pWnd;
	
	if ((nControlCount < 1) || (pControls == NULL))
	{
		AfxMessageBox("PanelManager::DisablePanel -- "
						"invalid control count or bad control pointer argument.");
		return FALSE;
	}

	for (nIndex = 0; nIndex < nControlCount; ++nIndex)
	{
		pWnd = m_pParent -> GetDlgItem(pControls[nIndex]);
		pWnd -> ShowWindow(SW_HIDE);
	}
	return TRUE;
}	
