/************************************************************************
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/




//	+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	+																		+
//	+	Simulator limitations:												+
//	+																		+
//	+	1.	The way the processor IACK cycle is simulated is such that only +
//	+		single-byte opcodes can be used as the response to an IACK from +
//	+		bus -- this is a limitation on the emulator fidelity, but since +
//	+		the H8's bus logic actually only syntesizes an "RST x"			+
//	+		instruction in reaction to an IACK cycle (and "RST x" is a		+
//	+		one-byte opcode), the infidelity of the simulator in this		+
//	+		instance is harmless.											+
//	+																		+
//	+	2.	Instruction timings are by no means the same as for the actual	+
//	+		processors (the timings cannot even be to scale, since each		+
//	+		processor uses a different number of clocks!); the number of	+
//	+		instructions per second in the emulator does not relate to the	+
//	+		actual speed of any of the physical processors.					+
//	+																		+
//	+	3.	The Z-80 emulation does not completely address the undocumented	+
//	+		behaviors of the Z-80, especially with respect to "duplicated",	+
//	+		"NOP'd" and just plain undocumented operations.  Nor is the		+
//	+		emulator's policy about these opcodes completely consistent --	+
//	+		in some cases, the undocumented "duplicate" or "NOP" is			+
//	+		executed; in other cases it faults as ILLEGAL_OP or				+
//	+		UNIMPLEMENTED.  This is not expected to be a problem, but will	+
//	+		be corrected anyway, as time permits.							+
//	+																		+
//	+	4.	Behavior with respect to flags may be buggy.  Especially the	+
//	+		unusual behaviors -- who actually depends on the fact that a	+
//	+		bit test instruction will set the AC flag, for example?			+
//	+																		+
//	+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//	============================================================================
//	
//	Notes on the Processor Simulation Object(s)
//
//	Why three microprocessors?  The H-8 originally shipped with a CPU card that
//	used an 8080.  Later, the H-89 was introduced and it had a Z-80.  Eventually
//	the H-8 had an optional CPU card that had a Z-80 as well.  And then there
//	was a flavor of HDOS for the Z-100 which had an 8085!  While I'm currently
//	only intending to emulate the H-8 with the Z-80 CPU card, I felt that the
//	three processors shared enough in common to make it worthwhile implementing
//	a core behavior as a base class and then developing three descendent classes
//	from it to deal with behavioral differences.  Boy, was I wrong!  But I've
//	gone too far down that road to reverse direction now.  And when I succeed,
//	it should be relatively easy to then use this code in emulating either the
//	baseline H-8 or to run a Z-100 HDOS emulator.
//
//	Besides the documented and undocumented extensions to the 8080 instruction
//	set, both the 8085 and (especially) the Z-80 execute some of the
//	instructions differently from the way the 8080 does, especially with respect
//	to flags.  Therefore, we maintain more than eight flag bits and we have to
//	declare DAA, PUSH PSW, POP PSW, CPO, CPE, RPO, RPE, JPO and JPE as "Varient
//	Opcodes", even though all three microprocessors do them.
//
//	TODO:	The three processors define three different, intersecting sets of
//	flag bits:
//
//		1.	The 8080 defines the PSW flag byte bits as follows:
//					7   6   5   4   3   2   1   0
//					S   Z   0   AC  0   P   1   C
//					|   |   |   |   |   |   |   |
//			sign ---+   |   |   |   |   |   |   +--- carry
//			zero -------+   |   |   |   |   +------- undefined
//			undefined ------+   |   |   +----------- parity
//			aux (DAA) carry ----+   +--------------- undefined
//			(the values in the "undefined" bit positions are what these bits
//			are set to when the flags are stored in a PUSH PSW operation.)
//
//		2.	The Z80 defines flag bit 1 as "AS" and uses a single "PV" flag bit
//			to encode either Parity or oVerflow in bit 2.  Bits 5 and 3 are
//			undocumented, but are set to bits 5 and 3 of the result of an
//			operation:
//					7   6   5   4   3   2   1   0
//					S   Z  Z5  AC  Z3  PV  AS   C
//					|   |   |   |   |   |   |   |
//			sign ---+   |   |   |   |   |   |   +--- carry
//			zero -------+   |   |   |   |   +------- add(0) / subtract(1)
//			D5 of result ---+   |   |   +----------- parity or overflow
//			aux (DAA) carry ----+   +--------------- D3 of result
//			The AS flag indicates whether the DAA instruction should adjust to
//			BCD for an ADD operation or a SUB operation.  (The 8080 and 8085
//			only correctly do BCD adjustment after ADD operations.)  The PV
//			flag indicates parity after a logical operation or overflow after
//			an arithmetic operation.  Z5 and Z3 are bits 5 and 3, respectively,
//			of the result of the previous operation.
//
//		3.	The 8085 Flags define two flag bits (at bit positions 5 and 1):
//
//					7   6   5   4   3   2   1   0
//					S   Z  X5  AC   0   P   V   C
//					|   |   |   |   |   |   |   |
//					|   |   |   |   |   |   |   +--- carry
//					|   |   |   |   |   |   +------- overflow
//					|   |   |   |   |   +----------- parity
//					|   |   |   |   +--------------- not used
//					|   |   |   +------------------- aux. carry
//					|   |   +----------------------- X5
//					|   +--------------------------- zero
//					+------------------------------- sign
//
//			Flag Computations for the 8085:
//
//				S 	= 	bit 7 of result
//				Z 	= 	result == 0
//				X5	=	(approximately) overflow/underflow on INX/DCX operation
//				AC	=	carry from bit 3 into bit 4
//				P	=	parity
//				V	=	arithmetic overflow or underflow on register operation
//				C	=	carry from bit 7
//
//				For the V bit,
//				  Overflow is defined as:
//					sign_of_operand_1 && sign_of_operand_2 
//						&& !sign_of_result ||
//					!sign_of_operand_1 && !sign_of_operand_2
//						&& sign_of_result
//
//				  and Underflow is defined as:
//					sign_of_operand_1 && !sign_of_operand_2
//						&& !sign_of_result ||
//					!sign_of_operand_1 && sign_of_operand_2
//						&& sign_of_result
//
//				The X5 bit is defined as
//
//				  after Addition:
//					sign_of_operand_1 && sign_of_operand_2 ||
//					sign_of_operand_1 && sign_of_result ||
//					sign_of_operand_2 && sign_of_result
//
//				  after Subtraction:
//					sign_of_operand_1 && !sign_of_operand_2 ||
//					sign_of_operand_1 && sign_of_result ||
//					!sign_of_operand_2 && sign_of_result
//
//			The X5 flag tested by the undocumented JX5 and JNX5 instructions;
//			the V flag is tested by the undocumented RSTV instruction.
//
//	Fortunately, you have to do something silly like "PUSH PSW; POP DE" to 
//	notice what the flags are doing.  Unfortunately, there may be programs out
//	there that do just exactly that while deciding which microprocessor they are
//	(or are willing to be) running on.
//
//	The PUSH PSW and POP PSW operations are coded to use the correct subset of
//	the twelve possible flags, even though only eight bits are actually used by
//	any processor.  By doing redundant flag computations and saving the results
//	in more than one bool, we avoid having to declare a much larger set of
//	varient opcodes.  But the opcodes that test the Parity flag must be declared
//	as varient so the Z80 can test the PV flag instead of the (unused) P flag.
//
//	The DAA instruction in all three processors will correctly convert the
//	result of an addition operation where the two operands are "packed BCD" back
//	into BCD.  The computation involves the AC flag (which represents a carry
//	from bit 3 into bit 4).  In the Z-80, the DAA instruction can also correctly
//	convert the result of a BCD subtraction back into BCD digits as well.  But
//	the DAA must use a different algorithm after a subtraction because the AC
//	flag now represents a borrow from bit 4 into bit 3.  The AS flag allows the
//	Z80 to differentiate between a DAA after an ADD or ADC versus a DAA after a
//	SUB or SBB.
//
//	Simulation of the Z80's INDR and OTDR opcodes is going to require some added
//	work -- the I/O operation will probably never be interruptable (I don't know
//	if it is on the chip, either.) and the simulated bus cycle logic will need
//	some more code to make it function.


//	04/13/2001: The emulation of the Z80 instruction set is only slightly more
//	extensive than is required for the Heathkit PAM-37 monitor code.  Further,
//	none of the Z80 opcodes not used by PAM-37 have been tested.
//
//	05/31/2001:	I've filled in more of the opcode map -- there's around 300
//	opcodes still to account for at this time -- but I've not tested any of the
//	newly-added opcodes yet.  I've also implemented some of the 8085 opcodes,
//	again with no testing done.


#include "stdafx.h"

#define	EXTERN	extern
#include "Virtual_h8.h"
#include "globals.h"
#include "processor.h"
#include "logicanalyzer.h"


//	The implementation of the opcodes makes for too much drek in this file, so
//	I moved the implementation code (which is methods of Processor) to another
//	file, included here:

#include "opcodes.inc"

//	============================================================================
//	Methods of the base class:

Processor :: Processor()
{

//	The following two _ASSERT macros ensure that the operating environment will
//	match the presumed sizes of data types.  This is critical to the emulation
//	of register and register-pair manipulations.  Note: run this code at least
//	once in DEBUG mode so the _ASSERTs get a chance to execute!

	_ASSERT(sizeof(PWORD) == 2);
	_ASSERT(sizeof(long) >= 2 * sizeof(PWORD));

//	The "run" status flag is deasserted on construction and asserted on reset.

	m_bRun = false;
}

Processor :: ~Processor()
{
}

void	Processor :: Reset()
{
}


int		Processor ::VarientOp(PBYTE )
{
	return UNIMPLEMENTED;
}


CString		Processor :: DisassembleVarientOp(PBYTE ,
												short int ,
												short int )
{
	return "ILLEGAL OP";
}


PBYTE	Processor :: Fetch()
{
	if (!m_bInterruptFetch)
	{

//	Get one byte from "memory" using PC as the address,
//	and incremenet PC.

		return m_MemMap.Read(PC++);
	}
	else
	{

//	Get one byte from the bus read data "register".

		m_bInterruptFetch = false;
		return m_nBusReadData;
	}
}

PWORD	Processor :: Fetch2()
{
	PWORD	nData;

//	Get two bytes from memory and store them as a PWORD.

	nData = m_MemMap.Read2(PC);
	PC += 2;
	return nData;
}

int	Processor :: Execute()
{
	int		nStatus;

	m_bIOWrite = false;

//	Check for completed I/O read.  Load the IO target register before doing
//	anything else.  (This will mean that the Z-80 block input operation is
//	not interruptable.  I don't know if a real Z-80 behaves that way or not.)

	if (m_bPendingRead && m_bIORead && !m_bIntAck)
	{

//	Deal with the I/O Read cycle.

		m_bPendingRead = false;
		m_bIORead = false;
		InputToRegister(m_nInputTarget, m_nBusReadData);
		if (m_bIORead && m_bPendingRead)
		{

//	We must be doing a Z-80 block input operation.

			return 0;
		}
	}

//	Next, deal with the interrupt flops and IACK stuff.

//	Background:
//	The 8080 had an external IREQ signal (m_bIntReq) and an internal interrupt
//	flip-flop (m_bInterruptFetch).  For m_bInterruptFetch to be set, m_bIntReq
//	and m_bInterruptsEnabled must both be set.  When m_bInterruptFetch is set,
//	m_bInterruptsEnabled gets reset and we set m_bIntAck.  We also want to put
//	the processor back into RUN mode.

	if (m_bIntReq && m_bInterruptsEnabled)
	{
		m_bInterruptsEnabled = false;
		m_bPendingInterruptEnable = false;
		m_bPendingRead = true;
		m_bIntAck = true;
		m_bM1 = false;
		m_bRun = true;

//	Normally, the 8080 would go on to do the current instruction and discover
//	the interrupt on the next M1 cycle.  We take a shortcut here that --
//	hopefully -- is harmless: we don't do the current instruction fetch and
//	therefore respond to the interrupt one opcode earlier.

		return 0;
	}

//	Next, deal with the processor in HALT state (!m_bRun): we do nothing except
//	return with m_bM1 false.

	if (!m_bRun)
	{
		m_bM1 = false;
		return 0;
	}
	if (m_bPendingRead && !m_bIORead && m_bIntAck)
	{
	
//	Deal with an IntAck response cycle.

		m_bPendingRead = false;
		m_bIntAck = false;
		m_bInterruptFetch = true;

//	Interrupt Fetch needs to be asserted for and will be deasserted by the
//	Step() function.

	}
	else
	{

//	Normal instruction cycle.  Clear all the unusual bus signal flags.

		m_bPendingRead = false;
		m_bIORead = false;
		m_bIntAck = false;
		m_bIOWrite = false;
		m_bInterruptFetch = false;
		m_bM1 = true;
	}

//	Okay, we can execute an instruction now.

	m_bRun = ((nStatus = Step()) == 0);
	if (nStatus == HALTED)
	{
		nStatus = 0;	//	Halt is a normal state, but we need to deassert Run.
	}
	return nStatus;
}


int	Processor :: Step()
{
	PBYTE		OP;
	static int	nIntEnableCount = 0;

	if (m_bPendingInterruptEnable)
	{

//	Interrupts are enabled 2 M1 cycles after the EI instruction.

		++nIntEnableCount;
		if (nIntEnableCount >= 2)
		{
			m_bInterruptsEnabled = true;
			m_bPendingInterruptEnable = false;
			nIntEnableCount = 0;
		}
	}
	if (m_bInterruptFetch)
	{
		m_bM1 = false;
		OP = Fetch();
		m_bIntAck = false;
		m_bInterruptFetch = false;

#ifdef _DEBUG

		if ((tfp != NULL) && (g_bLAM1CycleTrace))
		{
			fprintf(tfp, "M1 Cycle: From IACK, Op = %s\n",
					(LPCSTR) Octal(OP));
		}

#endif

	}
	else
	{
		m_bM1 = true;
		OP = Fetch();

#ifdef _DEBUG

		if ((tfp != NULL) && (g_bLAM1CycleTrace))
		{
			fprintf(tfp, "M1 Cycle: PC = %s, Op = %s\n",
					(LPCSTR) SplitOctal(PC - 1),
					(LPCSTR) Disassemble((PWORD)
											((PC - 1) & 0x00ffff)));
		}

#endif
		
	}
	m_nStatusOfVarientOp = 0;
	switch(OP)
	{
	case 0000:				//	nop
		NOP();
		break;
	case 0001:				//	lxi		b,++
		LXI_B();
		break;
	case 0002:				//	stax	b
		STAX_B();
		break;
	case 0003:				//	inx		b
		INX_B();
		break;
	case 0004:				//	inr		b
		INR_B();
		break;
	case 0005:	//	dcr		b
		DCR_B();
		break;
	case 0006:	//	mvi		b,+
		MVI_B();
		break;
	case 0007:	//	rlc
		RLC();
		break;
	case 0010:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	undocumented DSUB instruction in 8085s
				//	EX on Z-80
		VAR_010();
		break;
	case 0011:	//	dad		b
		DAD_B();
		break;
	case 0012:	//	ldax	b
		LDAX_B();
		break;
	case 0013:	//	dcx		b
		DCX_B();
		break;
	case 0014:	//	inr		c
		INR_C();
		break;
	case 0015:	//	dcr		c
		DCR_C();
		break;
	case 0016:	//	mvi		c,+
		MVI_C();
		break;
	case 0017:	//	rrc
		RRC();
		break;
	case 0020:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	undocumented ARHL instruction in 8085s
				//	DJNZ rel_disp on Z-80
		VAR_020();
		break;
	case 0021:	//	lxi		d,++
		LXI_D();
		break;
	case 0022:	//	stax	d
		STAX_D();
		break;
	case 0023:	//	inx		d
		INX_D();
		break;
	case 0024:	//	inr		d
		INR_D();
		break;
	case 0025:	//	dcr		d
		DCR_D();
		break;
	case 0026:	//	mvi		d,+
		MVI_D();
		break;
	case 0027:	//	ral
		RAL();
		break;
	case 0030:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	undocumented RDEL instruction in 8085s
				//	JR rel_disp on Z-80
		VAR_030();
		break;
	case 0031:	//	dad		d
		DAD_D();
		break;
	case 0032:	//	ldax	d
		LDAX_D();
		break;
	case 0033:	//	dcx		d
		DCX_D();
		break;
	case 0034:	//	inr		e
		INR_E();
		break;
	case 0035:	//	dcr		e
		DCR_E();
		break;
	case 0036:	//	mvi		e,+
		MVI_E();
		break;
	case 0037:	//	rar
		RAR();
		break;
	case 0040:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	RIM instruction in 8085s
				//	JRNZ rel_disp on Z-80
		VAR_040();
		break;
	case 0041:	//	lxi		h,++
		LXI_H();
		break;
	case 0042:	//	shld	++
		SHLD();
		break;
	case 0043:	//	inx		h
		INX_H();
		break;
	case 0044:	//	inr		h
		INR_H();
		break;
	case 0045:	//	dcr		h
		DCR_H();
		break;
	case 0046:	//	mvi		h,+
		MVI_H();
		break;
	case 0047:	//	DAA -- behaves differently on each processor
		DAA();
		break;
	case 0050:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	undocumented LDHI instruction in 8085s
				//	JRZ rel_disp on Z-80
		VAR_050();
		break;
	case 0051:	//	dad		h
		DAD_H();
		break;
	case 0052:	//	lhld	++
		LHLD();
		break;
	case 0053:	//	dcx		h
		DCX_H();
		break;
	case 0054:	//	inr		l
		INR_L();
		break;
	case 0055:	//	dcr		l
		DCR_L();
		break;
	case 0056:	//	mvi		l,+
		MVI_L();
		break;
	case 0057:	//	cma
		CMA();
		break;
	case 0060:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	SIM instruction in 8085s
				//	JRNC rel_disp on Z-80
		VAR_060();
		break;
	case 0061:	//	lxi		sp,++
		LXI_SP();
		break;
	case 0062:	//	sta		++
		STA();
		break;
	case 0063:	//	inx		sp
		INX_SP();
		break;
	case 0064:	//	inr		m
		INR_M();
		break;
	case 0065:	//	dcr		m
		DCR_M();
		break;
	case 0066:	//	mvi		m
		MVI_M();
		break;
	case 0067:	//	stc
		STC();
		break;
	case 0070:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	undocumented LDSI instruction in 8085s
				//	JRC rel_disp on Z-80
		VAR_070();
		break;
	case 0071:	//	dad		sp
		DAD_SP();
		break;
	case 0072:	//	lda		++
		LDA();
		break;
	case 0073:	//	dcx		sp
		DCX_SP();
		break;
	case 0074:	//	inr		a
		INR_A();
		break;
	case 0075:	//	dcr		a
		DCR_A();
		break;
	case 0076:	//	mvi		a
		MVI_A();
		break;
	case 0077:	//	cmc
		CMC();
		break;
	case 0100:	//	mov		b,b -- nop
		MOV_BB();
		break;
	case 0101:	//	mov		b,c
		MOV_BC();
		break;
	case 0102:	//	mov		b,d
		MOV_BD();
		break;
	case 0103:	//	mov		b,e
		MOV_BE();
		break;
	case 0104:	//	mov		b,h
		MOV_BH();
		break;
	case 0105:	//	mov		b,l
		MOV_BL();
		break;
	case 0106:	//	mov		b,m
		MOV_BM();
		break;
	case 0107:	//	mov		b,a
		MOV_BA();
		break;
	case 0110:	//	mov		c,b
		MOV_CB();
		break;
	case 0111:	//	mov		c,c
		MOV_CC();
		break;
	case 0112:	//	mov		c,d
		MOV_CD();
		break;
	case 0113:	//	mov		c,e
		MOV_CE();
		break;
	case 0114:	//	mov		c,h
		MOV_CH();
		break;
	case 0115:	//	mov		c,l
		MOV_CL();
		break;
	case 0116:	//	mov		c,m
		MOV_CM();
		break;
	case 0117:	//	mov		c,a
		MOV_CA();
		break;
	case 0120:	//	mov		d,b
		MOV_DB();
		break;
	case 0121:	//	mov		d,c
		MOV_DC();
		break;
	case 0122:	//	mov		d,d
		MOV_DD();
		break;
	case 0123:	//	mov		d,e
		MOV_DE();
		break;
	case 0124:	//	mov		d,h
		MOV_DH();
		break;
	case 0125:	//	mov		d,l
		MOV_DL();
		break;
	case 0126:	//	mov		d,m
		MOV_DM();
		break;
	case 0127:	//	mov		d,a
		MOV_DA();
		break;
	case 0130:	//	mov		e,b
		MOV_EB();
		break;
	case 0131:	//	mov		e,c
		MOV_EC();
		break;
	case 0132:	//	mov		e,d
		MOV_ED();
		break;
	case 0133:	//	mov		e,e
		MOV_EE();
		break;
	case 0134:	//	mov		e,h
		MOV_EH();
		break;
	case 0135:	//	mov		e,l
		MOV_EL();
		break;
	case 0136:	//	mov		e,m
		MOV_EM();
		break;
	case 0137:	//	mov		e,a
		MOV_EA();
		break;
	case 0140:	//	mov		h,b
		MOV_HB();
		break;
	case 0141:	//	mov		h,c
		MOV_HC();
		break;
	case 0142:	//	mov		h,d
		MOV_HD();
		break;
	case 0143:	//	mov		h,e
		MOV_HE();
		break;
	case 0144:	//	mov		h,h
		MOV_HH();
		break;
	case 0145:	//	mov		h,l
		MOV_HL();
		break;
	case 0146:	//	mov		h,m
		MOV_HM();
		break;
	case 0147:	//	mov		h,a
		MOV_HA();
		break;
	case 0150:	//	mov		l,b
		MOV_LB();
		break;
	case 0151:	//	mov		l,c
		MOV_LC();
		break;
	case 0152:	//	mov		l,d
		MOV_LD();
		break;
	case 0153:	//	mov		l,e
		MOV_LE();
		break;
	case 0154:	//	mov		l,h
		MOV_LH();
		break;
	case 0155:	//	mov		l,l
		MOV_LL();
		break;
	case 0156:	//	mov		l,m
		MOV_LM();
		break;
	case 0157:	//	mov		l,a
		MOV_LA();
		break;
	case 0160:	//	mov		m,b
		MOV_MB();
		break;
	case 0161:	//	mov		m,c
		MOV_MC();
		break;
	case 0162:	//	mov		m,d
		MOV_MD();
		break;
	case 0163:	//	mov		m,e
		MOV_ME();
		break;
	case 0164:	//	mov		m,h
		MOV_MH();
		break;
	case 0165:	//	mov		m,l
		MOV_ML();
		break;
	case 0166:	//	hlt
		HLT();
		break;
	case 0167:	//	mov		m,a
		MOV_MA();
		break;
	case 0170:	//	mov		a,b
		MOV_AB();
		break;
	case 0171:	//	mov		a,c
		MOV_AC();
		break;
	case 0172:	//	mov		a,d
		MOV_AD();
		break;
	case 0173:	//	mov		a,e
		MOV_AE();
		break;
	case 0174:	//	mov		a,h
		MOV_AH();
		break;
	case 0175:	//	mov		a,l
		MOV_AL();
		break;
	case 0176:	//	mov		a,m
		MOV_AM();
		break;
	case 0177:	//	mov		a,a
		MOV_AA();
		break;
	case 0200:	//	add		b
		ADD_B();
		break;
	case 0201:	//	add		c
		ADD_C();
		break;
	case 0202:	//	add		d
		ADD_D();
		break;
	case 0203:	//	add		e
		ADD_E();
		break;
	case 0204:	//	add		h
		ADD_H();
		break;
	case 0205:	//	add		l
		ADD_L();
		break;
	case 0206:	//	add		m
		ADD_M();
		break;
	case 0207:	//	add		a
		ADD_A();
		break;
	case 0210:	//	adc		b
		ADC_B();
		break;
	case 0211:	//	adc		c
		ADC_C();
		break;
	case 0212:	//	adc		d
		ADC_D();
		break;
	case 0213:	//	adc		e
		ADC_E();
		break;
	case 0214:	//	adc		h
		ADC_H();
		break;
	case 0215:	//	adc		l
		ADC_L();
		break;
	case 0216:	//	adc		m
		ADC_M();
		break;
	case 0217:	//	adc		a
		ADC_A();
		break;
	case 0220:	//	sub		b
		SUB_B();
		break;
	case 0221:	//	sub		c
		SUB_C();
		break;
	case 0222:	//	sub		d
		SUB_D();
		break;
	case 0223:	//	sub		e
		SUB_E();
		break;
	case 0224:	//	sub		h
		SUB_H();
		break;
	case 0225:	//	sub		l
		SUB_L();
		break;
	case 0226:	//	sub		m
		SUB_M();
		break;
	case 0227:	//	sub		a
		SUB_A();
		break;
	case 0230:	//	sbb		b
		SBB_B();
		break;
	case 0231:	//	sbb		c
		SBB_C();
		break;
	case 0232:	//	sbb		d
		SBB_D();
		break;
	case 0233:	//	sbb		e
		SBB_E();
		break;
	case 0234:	//	sbb		h
		SBB_H();
		break;
	case 0235:	//	sbb		l
		SBB_L();
		break;
	case 0236:	//	sbb		m
		SBB_M();
		break;
	case 0237:	//	sbb		a
		SBB_A();
		break;
	case 0240:	//	ana		b
		ANA_B();
		break;
	case 0241:	//	ana		c
		ANA_C();
		break;
	case 0242:	//	ana		d
		ANA_D();
		break;
	case 0243:	//	ana		e
		ANA_E();
		break;
	case 0244:	//	ana		h
		ANA_H();
		break;
	case 0245:	//	ana		l
		ANA_L();
		break;
	case 0246:	//	ana		m
		ANA_M();
		break;
	case 0247:	//	ana		a
		ANA_A();
		break;
	case 0250:	//	xra		b
		XRA_B();
		break;
	case 0251:	//	xra		c
		XRA_C();
		break;
	case 0252:	//	xra		d
		XRA_D();
		break;
	case 0253:	//	xra		e
		XRA_E();
		break;
	case 0254:	//	xra		h
		XRA_H();
		break;
	case 0255:	//	xra		l
		XRA_L();
		break;
	case 0256:	//	xra		m
		XRA_M();
		break;
	case 0257:	//	xra		a
		XRA_A();
		break;
	case 0260:	//	ora		b
		ORA_B();
		break;
	case 0261:	//	ora		c
		ORA_C();
		break;
	case 0262:	//	ora		d
		ORA_D();
		break;
	case 0263:	//	ora		e
		ORA_E();
		break;
	case 0264:	//	ora		h
		ORA_H();
		break;
	case 0265:	//	ora		l
		ORA_L();
		break;
	case 0266:	//	ora		m
		ORA_M();
		break;
	case 0267:	//	ora		a
		ORA_A();
		break;
	case 0270:	//	cmp		b
		CMP_B();
		break;
	case 0271:	//	cmp		c
		CMP_C();
		break;
	case 0272:	//	cmp		d
		CMP_D();
		break;
	case 0273:	//	cmp		e
		CMP_E();
		break;
	case 0274:	//	cmp		h
		CMP_H();
		break;
	case 0275:	//	cmp		l
		CMP_L();
		break;
	case 0276:	//	cmp		m
		CMP_M();
		break;
	case 0277:	//	cmp		a
		CMP_A();
		break;
	case 0300:	//	rnz
		RNZ();
		break;
	case 0301:	//	pop		b
		POP_B();
		break;
	case 0302:	//	jnz		++
		JNZ();
		break;
	case 0303:	//	jmp		++
		JMP();
		break;
	case 0304:	//	cnz		++
		CNZ();
		break;
	case 0305:	//	push	b
		PUSH_B();
		break;
	case 0306:	//	adi		+
		ADI();
		break;
	case 0307:	//	rst		0
		RST_0();
		break;
	case 0310:	//	rz
		RZ();
		break;
	case 0311:	//	ret
		RET();
		break;
	case 0312:	//	jz		++
		JZ();
		break;
	case 0313:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	undocumented RSTV instruction in 8085s
				//	prefix for a group of multi-byte opcodes on Z-80
		VAR_313();
		break;
	case 0314:	//	cz		++
		CZ();
		break;
	case 0315:	//	call	++
		CALL();
		break;
	case 0316:	//	aci		+
		ACI();
		break;
	case 0317:	//	rst		1
		RST_1();
		break;
	case 0320:	//	rnc
		RNC();
		break;
	case 0321:	//	pop		d
		POP_D();
		break;
	case 0322:	//	jnc		++
		JNC();
		break;
	case 0323:	//	out		+
		IO_OUT();
		break;
	case 0324:	//	cnc		++
		CNC();
		break;
	case 0325:	//	push	d
		PUSH_D();
		break;
	case 0326:	//	sui		+
		SUI();
		break;
	case 0327:	//	rst		2
		RST_2();
		break;
	case 0330:	//	rc
		RC();
		break;
	case 0331:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	undocumented SHLX instruction in 8085s
				//	EXX on Z-80
		VAR_331();
		break;
	case 0332:	//	jc		++
		JC();
		break;
	case 0333:	//	in		+
		IO_IN();
		break;
	case 0334:	//	cc		++
		CC();
		break;
	case 0335:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	undocumented JNX5 instruction in 8085s
				//	prefix for a multi-byte opcode group on Z-80
		VAR_335();
		break;
	case 0336:	//	sbi		+
		SBI();
		break;
	case 0337:	//	rst		3
		RST_3();
		break;
	case 0340:	//	rpo
		RPO();
		break;
	case 0341:	//	pop		h
		POP_H();
		break;
	case 0342:	//	jpo		++
		JPO();
		break;
	case 0343:	//	xthl
		XTHL();
		break;
	case 0344:	//	cpo		++
		CPO();
		break;
	case 0345:	//	push	h
		PUSH_H();
		break;
	case 0346:	//	ani		+
		ANI();
		break;
	case 0347:	//	rst		4
		RST_4();
		break;
	case 0350:	//	rpe
		RPE();
		break;
	case 0351:	//	pchl
		PCHL();
		break;
	case 0352:	//	jpe		++
		JPE();
		break;
	case 0353:	//	xchg
		XCHG();
		break;
	case 0354:	//	cpe		++
		CPE();
		break;
	case 0355:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	undocumented LHLX instruction in 8085s
				//	prefix for a multi-byte opcode group on Z-80
		VAR_355();
		break;
	case 0356:	//	xri		+
		XRI();
		break;
	case 0357:	//	rst		5
		RST_5();
		break;
	case 0360:	//	rp
		RP();
		break;
	case 0361:	//	pop		psw -- processor dependent values!
		POP_PSW();
		break;
	case 0362:	//	jp		++
		JP();
		break;
	case 0363:	//	di
		DI();
		break;
	case 0364:	//	cp		++
		CP();
		break;
	case 0365:	//	push	psw -- processor dependent values!
		PUSH_PSW();
		break;
	case 0366:	//	ori		+
		ORI();
		break;
	case 0367:	//	rst		6
		RST_6();
		break;
	case 0370:	//	rm
		RM();
		break;
	case 0371:	//	sphl
		SPHL();
		break;
	case 0372:	//	jm		++
		JM();
		break;
	case 0373:	//	ei
		EI();
		break;
	case 0374:	//	cm		++
		CM();
		break;
	case 0375:	//	PROCESSOR-DEPENDENT INSTRUCTION
				//	illegal in 8080s
				//	undocumented JX5 instruction in 8085s
				//	prefix for a multi-byte opcode group on Z-80
		VAR_375();
		break;
	case 0376:	//	cpi		+
		CPI();
		break;
	case 0377:	//	rst		7
		RST_7();
		break;
	}
	return m_nStatusOfVarientOp;
}


//	Disassembler for 8080
//	Opcodes by Numeric Value:
//
//		000						NOP
//		001						LXI		B,++
//		002						STAX	B
//		003						INX		B
//		004						INR		B
//		005						DCR		B
//		006						MVI		B,+
//		007						RLC
//		010						PROCESSOR-DEPENDENT INSTRUCTION
//		011						DAD		B
//		012						LDAX	B
//		013						DCX		B
//		014						INR		C
//		015						DCR		C
//		016						MVI		C,+
//		017						RRC
//		020						PROCESSOR-DEPENDENT INSTRUCTION
//		021						LXI		D,++
//		022						STAX	D
//		023						INX		D
//		024						INR		D
//		025						DCR		D
//		026						MVI		D,+
//		027						RAL
//		030						PROCESSOR-DEPENDENT INSTRUCTION
//		031						DAD		D
//		032						LDAX	D
//		033						DCX		D
//		034						INR		E
//		035						DCR		E
//		036						MVI		E,+
//		037						RAR
//		040						PROCESSOR-DEPENDENT INSTRUCTION
//		041						LXI		H,++
//		042						SHLD	++
//		043						INX		H
//		044						INR		H
//		045						DCR		H
//		046						MVI		H,+
//		047						DAA
//		050						PROCESSOR-DEPENDENT INSTRUCTION
//		051						DAD		H
//		052						LHLD	++
//		053						DCX		H
//		054						INR		L
//		055						DCR		L
//		056						MVI		L,+
//		057						CMA
//		060						PROCESSOR-DEPENDENT INSTRUCTION
//		061						LXI		SP,++
//		062						STA		++
//		063						INX		SP
//		064						INR		M
//		065						DCR		M
//		066						MVI		M
//		067						STC
//		070						PROCESSOR-DEPENDENT INSTRUCTION
//		071						DAD		SP
//		072						LDA		++
//		073						DCX		SP
//		074						INR		A
//		075						DCR		A
//		076						MVI		A
//		077						CMC
//		100						MOV		B,B
//		101						MOV		B,C
//		102						MOV		B,D
//		103						MOV		B,E
//		104						MOV		B,H
//		105						MOV		B,L
//		106						MOV		B,M
//		107						MOV		B,A
//		110						MOV		C,B
//		111						MOV		C,C
//		112						MOV		C,D
//		113						MOV		C,E
//		114						MOV		C,H
//		115						MOV		C,L
//		116						MOV		C,M
//		117						MOV		C,A
//		120						MOV		D,B
//		121						MOV		D,C
//		122						MOV		D,D
//		123						MOV		D,E
//		124						MOV		D,H
//		125						MOV		D,L
//		126						MOV		D,M
//		127						MOV		D,A
//		130						MOV		E,B
//		131						MOV		E,C
//		132						MOV		E,D
//		133						MOV		E,E
//		134						MOV		E,H
//		135						MOV		E,L
//		136						MOV		E,M
//		137						MOV		E,A
//		140						MOV		H,B
//		141						MOV		H,C
//		142						MOV		H,D
//		143						MOV		H,E
//		144						MOV		H,H
//		145						MOV		H,L
//		146						MOV		H,M
//		147						MOV		H,A
//		150						MOV		L,B
//		151						MOV		L,C
//		152						MOV		L,D
//		153						MOV		L,E
//		154						MOV		L,H
//		155						MOV		L,L
//		156						MOV		L,M
//		157						MOV		L,A
//		160						MOV		M,B
//		161						MOV		M,C
//		162						MOV		M,D
//		163						MOV		M,E
//		164						MOV		M,H
//		165						MOV		M,L
//		166						HLT
//		167						MOV		M,A
//		170						MOV		A,B
//		171						MOV		A,C
//		172						MOV		A,D
//		173						MOV		A,E
//		174						MOV		A,H
//		175						MOV		A,L
//		176						MOV		A,M
//		177						MOV		A,A
//		200						ADD		B
//		201						ADD		C
//		202						ADD		D
//		203						ADD		E
//		204						ADD		H
//		205						ADD		L
//		206						ADD		M
//		207						ADD		A
//		210						ADC		B
//		211						ADC		C
//		212						ADC		D
//		213						ADC		E
//		214						ADC		H
//		215						ADC		L
//		216						ADC		M
//		217						ADC		A
//		220						SUB		B
//		221						SUB		C
//		222						SUB		D
//		223						SUB		E
//		224						SUB		H
//		225						SUB		L
//		226						SUB		M
//		227						SUB		A
//		230						SBB		B
//		231						SBB		C
//		232						SBB		D
//		233						SBB		E
//		234						SBB		H
//		235						SBB		L
//		236						SBB		M
//		237						SBB		A
//		240						ANA		B
//		241						ANA		C
//		242						ANA		D
//		243						ANA		E
//		244						ANA		H
//		245						ANA		L
//		246						ANA		M
//		247						ANA		A
//		250						XRA		B
//		251						XRA		C
//		252						XRA		D
//		253						XRA		E
//		254						XRA		H
//		255						XRA		L
//		256						XRA		M
//		257						XRA		A
//		260						ORA		B
//		261						ORA		C
//		262						ORA		D
//		263						ORA		E
//		264						ORA		H
//		265						ORA		L
//		266						ORA		M
//		267						ORA		A
//		270						CMP		B
//		271						CMP		C
//		272						CMP		D
//		273						CMP		E
//		274						CMP		H
//		275						CMP		L
//		276						CMP		M
//		277						CMP		A
//		300						RNZ
//		301						POP		B
//		302						JNZ		++
//		303						JMP		++
//		304						CNZ		++
//		305						PUSH	B
//		306						ADI		+
//		307						RST		0
//		310						RZ
//		311						RET
//		312						JZ		++
//		313						PROCESSOR-DEPENDENT INSTRUCTION
//		314						CZ		++
//		315						CALL	++
//		316						ACI		+
//		317						RST		1
//		320						RNC
//		321						POP		D
//		322						JNC		++
//		323						OUT		+
//		324						CNC		++
//		325						PUSH	D
//		326						SUI		+
//		327						RST		2
//		330						RC
//		331						PROCESSOR-DEPENDENT INSTRUCTION
//		332						JC		++
//		333						IN		+
//		334						CC		++
//		335						PROCESSOR-DEPENDENT INSTRUCTION
//		336						SBI		+
//		337						RST		3
//		340						RPO
//		341						POP		H
//		342						JPO		++
//		343						XTHL
//		344						CPO		++
//		345						PUSH	H
//		346						ANI		+
//		347						RST		4
//		350						RPE
//		351						PCHL
//		352						JPE		++
//		353						XCHG
//		354						CPE		++
//		355						PROCESSOR-DEPENDENT INSTRUCTION
//		356						XRI		+
//		357						RST		5
//		360						RP
//		361						POP		PSW
//		362						JP		++
//		363						DI
//		364						CP		++
//		365						PUSH	PSW
//		366						ORI		+
//		367						RST		6
//		370						RM
//		371						SPHL
//		372						JM		++
//		373						EI
//		374						CM		++
//		375						PROCESSOR-DEPENDENT INSTRUCTION
//		376						CPI		+
//		377						RST		7



CString	Processor :: Disassemble(PWORD nPC)
{
	CString		strCode;
	PBYTE		nOp;
	union
	{
		PBYTE		c[2];	// c[0] is low-order, c[1] is high-order
		PWORD		w;
	} operand, xop;

	nOp = m_MemMap.Read(nPC++);
	operand.c[0] = m_MemMap.Read(nPC++);
	operand.c[1] = m_MemMap.Read(nPC);
	xop.c[0] = m_MemMap.Read((PWORD)(nPC + 1));
	xop.c[1] = m_MemMap.Read((PWORD)(nPC + 2));
	switch(nOp)
	{
	case 000:
		strCode = "NOP";
		break;
	case 001:
		strCode.Format("LXI   B,%s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 002:
		strCode = "STAX  B";
		break;
	case 003:
		strCode = "INX   B";
		break;
	case 004:
		strCode = "INR   B";
		break;
	case 005:
		strCode = "DCR   B";
		break;
	case 006:
		strCode.Format("MVI   B,%s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 007:
		strCode = "RLC";
		break;
	case 010:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 011:
		strCode = "DAD   B";
		break;
	case 012:
		strCode = "LDAX  B";
		break;
	case 013:
		strCode = "DCX   B";
		break;
	case 014:
		strCode = "INR   C";
		break;
	case 015:
		strCode = "DCR   C";
		break;
	case 016:
		strCode.Format("MVI   C,%s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 017:
		strCode = "RRC";
		break;
	case 020:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 021:
		strCode.Format("LXI   D,%s", SplitOctal(operand.w));
		break;
	case 022:
		strCode = "STAX  D";
		break;
	case 023:
		strCode = "INX   D";
		break;
	case 024:
		strCode = "INR   D";
		break;
	case 025:
		strCode = "DCR   D";
		break;
	case 026:
		strCode.Format("MVI   D,%s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 027:
		strCode = "RAL";
		break;
	case 030:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 031:
		strCode = "DAD   D";
		break;
	case 032:
		strCode = "LDAX  D";
		break;
	case 033:
		strCode = "DCX   D";
		break;
	case 034:
		strCode = "INR   E";
		break;
	case 035:
		strCode = "DCR   E";
		break;
	case 036:
		strCode.Format("MVI   E,%s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 037:
		strCode = "RAR";
		break;
	case 040:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 041:
		strCode.Format("LXI   H,%s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 042:
		strCode.Format("SHLD  %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 043:
		strCode = "INX   H";
		break;
	case 044:
		strCode = "INR   H";
		break;
	case 045:
		strCode = "DCR   H";
		break;
	case 046:
		strCode.Format("MVI   H,%s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 047:
		strCode = "DAA";
		break;
	case 050:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 051:
		strCode = "DAD   H";
		break;
	case 052:
		strCode.Format("LHLD  %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 053:
		strCode = "DCX   H";
		break;
	case 054:
		strCode = "INR   L";
		break;
	case 055:
		strCode = "DCR   L";
		break;
	case 056:
		strCode.Format("MVI   L,%s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 057:
		strCode = "CMA";
		break;
	case 060:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 061:
		strCode.Format("LXI   SP,%s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 062:
		strCode.Format("STA   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 063:
		strCode = "INX   SP";
		break;
	case 064:
		strCode = "INR   M";
		break;
	case 065:
		strCode = "DCR   M";
		break;
	case 066:
		strCode = "MVI   M";
		break;
	case 067:
		strCode = "STC";
		break;
	case 070:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 071:
		strCode = "DAD   SP";
		break;
	case 072:
		strCode.Format("LDA   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 073:
		strCode = "DCX   SP";
		break;
	case 074:
		strCode = "INR   A";
		break;
	case 075:
		strCode = "DCR   A";
		break;
	case 076:
		strCode = "MVI   A";
		break;
	case 077:
		strCode = "CMC";
		break;
	case 0100:
		strCode = "MOV   B,B";
		break;
	case 0101:
		strCode = "MOV   B,C";
		break;
	case 0102:
		strCode = "MOV   B,D";
		break;
	case 0103:
		strCode = "MOV   B,E";
		break;
	case 0104:
		strCode = "MOV   B,H";
		break;
	case 0105:
		strCode = "MOV   B,L";
		break;
	case 0106:
		strCode = "MOV   B,M";
		break;
	case 0107:
		strCode = "MOV   B,A";
		break;
	case 0110:
		strCode = "MOV   C,B";
		break;
	case 0111:
		strCode = "MOV   C,C";
		break;
	case 0112:
		strCode = "MOV   C,D";
		break;
	case 0113:
		strCode = "MOV   C,E";
		break;
	case 0114:
		strCode = "MOV   C,H";
		break;
	case 0115:
		strCode = "MOV   C,L";
		break;
	case 0116:
		strCode = "MOV   C,M";
		break;
	case 0117:
		strCode = "MOV   C,A";
		break;
	case 0120:
		strCode = "MOV   D,B";
		break;
	case 0121:
		strCode = "MOV   D,C";
		break;
	case 0122:
		strCode = "MOV   D,D";
		break;
	case 0123:
		strCode = "MOV   D,E";
		break;
	case 0124:
		strCode = "MOV   D,H";
		break;
	case 0125:
		strCode = "MOV   D,L";
		break;
	case 0126:
		strCode = "MOV   D,M";
		break;
	case 0127:
		strCode = "MOV   D,A";
		break;
	case 0130:
		strCode = "MOV   E,B";
		break;
	case 0131:
		strCode = "MOV   E,C";
		break;
	case 0132:
		strCode = "MOV   E,D";
		break;
	case 0133:
		strCode = "MOV   E,E";
		break;
	case 0134:
		strCode = "MOV   E,H";
		break;
	case 0135:
		strCode = "MOV   E,L";
		break;
	case 0136:
		strCode = "MOV   E,M";
		break;
	case 0137:
		strCode = "MOV   E,A";
		break;
	case 0140:
		strCode = "MOV   H,B";
		break;
	case 0141:
		strCode = "MOV   H,C";
		break;
	case 0142:
		strCode = "MOV   H,D";
		break;
	case 0143:
		strCode = "MOV   H,E";
		break;
	case 0144:
		strCode = "MOV   H,H";
		break;
	case 0145:
		strCode = "MOV   H,L";
		break;
	case 0146:
		strCode = "MOV   H,M";
		break;
	case 0147:
		strCode = "MOV   H,A";
		break;
	case 0150:
		strCode = "MOV   L,B";
		break;
	case 0151:
		strCode = "MOV   L,C";
		break;
	case 0152:
		strCode = "MOV   L,D";
		break;
	case 0153:
		strCode = "MOV   L,E";
		break;
	case 0154:
		strCode = "MOV   L,H";
		break;
	case 0155:
		strCode = "MOV   L,L";
		break;
	case 0156:
		strCode = "MOV   L,M";
		break;
	case 0157:
		strCode = "MOV   L,A";
		break;
	case 0160:
		strCode = "MOV   M,B";
		break;
	case 0161:
		strCode = "MOV   M,C";
		break;
	case 0162:
		strCode = "MOV   M,D";
		break;
	case 0163:
		strCode = "MOV   M,E";
		break;
	case 0164:
		strCode = "MOV   M,H";
		break;
	case 0165:
		strCode = "MOV   M,L";
		break;
	case 0166:
		strCode = "HLT";
		break;
	case 0167:
		strCode = "MOV   M,A";
		break;
	case 0170:
		strCode = "MOV   A,B";
		break;
	case 0171:
		strCode = "MOV   A,C";
		break;
	case 0172:
		strCode = "MOV   A,D";
		break;
	case 0173:
		strCode = "MOV   A,E";
		break;
	case 0174:
		strCode = "MOV   A,H";
		break;
	case 0175:
		strCode = "MOV   A,L";
		break;
	case 0176:
		strCode = "MOV   A,M";
		break;
	case 0177:
		strCode = "MOV   A,A";
		break;
	case 0200:
		strCode = "ADD   B";
		break;
	case 0201:
		strCode = "ADD   C";
		break;
	case 0202:
		strCode = "ADD   D";
		break;
	case 0203:
		strCode = "ADD   E";
		break;
	case 0204:
		strCode = "ADD   H";
		break;
	case 0205:
		strCode = "ADD   L";
		break;
	case 0206:
		strCode = "ADD   M";
		break;
	case 0207:
		strCode = "ADD   A";
		break;
	case 0210:
		strCode = "ADC   B";
		break;
	case 0211:
		strCode = "ADC   C";
		break;
	case 0212:
		strCode = "ADC   D";
		break;
	case 0213:
		strCode = "ADC   E";
		break;
	case 0214:
		strCode = "ADC   H";
		break;
	case 0215:
		strCode = "ADC   L";
		break;
	case 0216:
		strCode = "ADC   M";
		break;
	case 0217:
		strCode = "ADC   A";
		break;
	case 0220:
		strCode = "SUB   B";
		break;
	case 0221:
		strCode = "SUB   C";
		break;
	case 0222:
		strCode = "SUB   D";
		break;
	case 0223:
		strCode = "SUB   E";
		break;
	case 0224:
		strCode = "SUB   H";
		break;
	case 0225:
		strCode = "SUB   L";
		break;
	case 0226:
		strCode = "SUB   M";
		break;
	case 0227:
		strCode = "SUB   A";
		break;
	case 0230:
		strCode = "SBB   B";
		break;
	case 0231:
		strCode = "SBB   C";
		break;
	case 0232:
		strCode = "SBB   D";
		break;
	case 0233:
		strCode = "SBB   E";
		break;
	case 0234:
		strCode = "SBB   H";
		break;
	case 0235:
		strCode = "SBB   L";
		break;
	case 0236:
		strCode = "SBB   M";
		break;
	case 0237:
		strCode = "SBB   A";
		break;
	case 0240:
		strCode = "ANA   B";
		break;
	case 0241:
		strCode = "ANA   C";
		break;
	case 0242:
		strCode = "ANA   D";
		break;
	case 0243:
		strCode = "ANA   E";
		break;
	case 0244:
		strCode = "ANA   H";
		break;
	case 0245:
		strCode = "ANA   L";
		break;
	case 0246:
		strCode = "ANA   M";
		break;
	case 0247:
		strCode = "ANA   A";
		break;
	case 0250:
		strCode = "XRA   B";
		break;
	case 0251:
		strCode = "XRA   C";
		break;
	case 0252:
		strCode = "XRA   D";
		break;
	case 0253:
		strCode = "XRA   E";
		break;
	case 0254:
		strCode = "XRA   H";
		break;
	case 0255:
		strCode = "XRA   L";
		break;
	case 0256:
		strCode = "XRA   M";
		break;
	case 0257:
		strCode = "XRA   A";
		break;
	case 0260:
		strCode = "ORA   B";
		break;
	case 0261:
		strCode = "ORA   C";
		break;
	case 0262:
		strCode = "ORA   D";
		break;
	case 0263:
		strCode = "ORA   E";
		break;
	case 0264:
		strCode = "ORA   H";
		break;
	case 0265:
		strCode = "ORA   L";
		break;
	case 0266:
		strCode = "ORA   M";
		break;
	case 0267:
		strCode = "ORA   A";
		break;
	case 0270:
		strCode = "CMP   B";
		break;
	case 0271:
		strCode = "CMP   C";
		break;
	case 0272:
		strCode = "CMP   D";
		break;
	case 0273:
		strCode = "CMP   E";
		break;
	case 0274:
		strCode = "CMP   H";
		break;
	case 0275:
		strCode = "CMP   L";
		break;
	case 0276:
		strCode = "CMP   M";
		break;
	case 0277:
		strCode = "CMP   A";
		break;
	case 0300:
		strCode = "RNZ";
		break;
	case 0301:
		strCode = "POP   B";
		break;
	case 0302:
		strCode.Format("JNZ   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0303:
		strCode.Format("JMP   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0304:
		strCode.Format("CNZ   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0305:
		strCode = "PUSH  B";
		break;
	case 0306:
		strCode.Format("ADI   %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0307:
		strCode = "RST   0";
		break;
	case 0310:
		strCode = "RZ";
		break;
	case 0311:
		strCode = "RET";
		break;
	case 0312:
		strCode.Format("JZ    %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0313:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 0314:
		strCode.Format("CZ    %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0315:
		strCode.Format("CALL  %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0316:
		strCode.Format("ACI   %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0317:
		strCode = "RST    1";
		break;
	case 0320:
		strCode = "RNC";
		break;
	case 0321:
		strCode = "POP   D";
		break;
	case 0322:
		strCode.Format("JNC   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0323:
		strCode.Format("OUT   %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0324:
		strCode.Format("CNC   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0325:
		strCode = "PUSH  D";
		break;
	case 0326:
		strCode.Format("SUI   %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0327:
		strCode = "RST   2";
		break;
	case 0330:
		strCode = "RC";
		break;
	case 0331:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 0332:
		strCode.Format("JC    %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0333:
		strCode.Format("IN    %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0334:
		strCode.Format("CC    %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0335:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 0336:
		strCode.Format("SBI   %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0337:
		strCode = "RST   3";
		break;
	case 0340:
		strCode = "RPO";
		break;
	case 0341:
		strCode = "POP   H";
		break;
	case 0342:
		strCode.Format("JPO   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0343:
		strCode = "XTHL";
		break;
	case 0344:
		strCode.Format("CPO   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0345:
		strCode = "PUSH	H";
		break;
	case 0346:
		strCode.Format("ANI   %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0347:
		strCode = "RST   4";
		break;
	case 0350:
		strCode = "RPE";
		break;
	case 0351:
		strCode = "PCHL";
		break;
	case 0352:
		strCode.Format("JPE   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0353:
		strCode = "XCHG";
		break;
	case 0354:
		strCode.Format("CPE   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0355:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 0356:
		strCode.Format("XRI   %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0357:
		strCode = "RST   5";
		break;
	case 0360:
		strCode = "RP";
		break;
	case 0361:
		strCode = "POP   PSW";
		break;
	case 0362:
		strCode.Format("JP    %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0363:
		strCode = "DI";
		break;
	case 0364:
		strCode.Format("CP    %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0365:
		strCode = "PUSH	PSW";
		break;
	case 0366:
		strCode.Format("ORI   %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0367:
		strCode = "RST   6";
		break;
	case 0370:
		strCode = "RM";
		break;
	case 0371:
		strCode = "SPHL";
		break;
	case 0372:
		strCode.Format("JM    %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0373:
		strCode = "EI";
		break;
	case 0374:
		strCode.Format("CM    %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0375:
		strCode = DisassembleVarientOp(nOp, operand.w, xop.w);
		break;
	case 0376:
		strCode.Format("CPI   %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0377:
		strCode = "RST   7";
		break;
	default:
		strCode.Format("Unrecognized %s %s %s",
						(LPCSTR) Octal(nOp),
						(LPCSTR) Octal(operand.c[0]),
						(LPCSTR) Octal(operand.c[1]));
		break;
	}
	return strCode;
}


//	============================================================================
//	Methods of the Z-80 emulator:

Z80 :: Z80()
{
	Reset();

//	The "run" status flag is deasserted on construction and asserted on reset.

	m_bRun =					false;
}

Z80 :: ~Z80()
{
}


void	Z80 :: Reset()
{

//	Ensure that the assumptions about data type sizes are correct.
	
	ASSERT(sizeof(PWORD) == 2);
	ASSERT(sizeof(int) == 4);

//	Initialize the registers and flags.

	CY =						false;		//	TODO: check this
	PV =						true;		//	TODO: check this
	AS =						false;		//	TODO: check this
	Z3 =						false;		//	TODO: check this
	AC =						false;		//	TODO: check this
	Z5 =						false;		//	TODO: check this
	Z =							true;		//	TODO: check this
	S =							false;		//	TODO: check this
	PC =						0;
	SP =						(PWORD) 0x0FFFF;
	A =							0;
	B =							0;
	C =							0;
	D =							0;
	E =							0;
	H =							0;
	L =							0;
	CY1 =						CY;
	AS1 =						AS;
	PV1 =						PV;
	Z31 =						Z3;
	AC1 =						AC;
	Z51 =						Z5;
	Z1 =						Z;
	S1 =						S;
	A1 =						0;
	B1 =						0;
	C1 =						0;
	D1 =						0;
	E1 =						0;
	H1 =						0;
	L1 =						0;
	I =							0;
	R =							0;
	IX =						0;
	IY =						0;

//	Initialize the interrupt enable and interrupt fetch flags.

	m_bInterruptFetch =			false;
	m_bPendingInterruptEnable =	false;

//	Initialize the address and data bus:

	m_nBusAddress =				0;
	m_nBusReadData =			0;
	m_nBusWriteData =			0;

//	Initialize the control signals that govern IO read and IO write cycles:

	m_bIORead =					false;
	m_bIOWrite =				false;
	m_bPendingRead =			false;

//	Initialize control signals that govern interrupt request and acknowledge

	m_bIntReq =					false;
	m_bIntAck =					false;

//	Initialize status signals

	m_bRun =					true;
	m_bInterruptsEnabled =		false;

//	These signals are not "connected".

	m_bSOUT =					false;
	m_bSIN =					false;
	m_bNMI =					false;
	m_bRST5_5 =					false;
	m_bRST6_5 =					false;
	m_bRST7_5 =					false;
}



int		Z80 :: 	VarientOp(PBYTE nOP)
{

//	IMPLEMENTATION BUG: Cannot be used as an INTERRUPT Instruction

	union
	{
		PBYTE		c[2];	// c[0] is low-order, c[1] is high-order
		PWORD		w;
	}					operand, xop;
	PFLAG				bFlag;
	int					nStatus = 0;
	PBYTE				nData;

	switch(nOP)
	{

//	The following opcodes behave differently on the Z80:

	case 0047:	//	daa
		operand.c[0] = (PBYTE)(A & 0x0F);
		if (!AS)
		{

//	1.	If the least signifiant 4 bits are greater than 9 or if AC is true, add
//		6 to A.
//	2.	Now if the most significant 4 bits are greater than 9 or if CY is true,
//		add 0x60 to A.
//	Affects all flags except AS.

			if ((operand.c[0] > 9) || (AC))
			{
				operand.c[0] += 6;
				operand.c[1] = A;
				if ((operand.c[1] + 6) > 0x00FF) CY = true;
				A += 6;
			}
			if (operand.c[0] > 0x0F) AC = true;
			else AC = false;
			operand.c[1] = (char)((A >> 4) & 0x0F);
			if ((operand.c[1] > 9) || (CY))
			{
				operand.c[1] += 6;
				if (operand.c[1] > 0x0F) CY = true;
				else CY = false;
				A += 0x60;
			}
			Z = (A == 0);
			S = (A & 0x0080 != 0);
			PV = Parity(A);
		}
		else
		{

//	1.	If the least signifiant 4 bits are greater than 9 or if AC is true,
//		subtract 6 from A.
//	2.	Now if the most significant 4 bits are greater than 9 or if CY is true,
//		subtract 0x60 from A.
//	Affects all flags except AS.

			if ((operand.c[0] > 9) || (AC))
			{
				operand.c[0] -= 6;
				operand.c[1] = A;
				if ((operand.c[1] - 6) & 0x0FF00) CY = true;
				A -= 6;
				A = (char)(A & 0x00FF);
			}
			if (operand.c[0] & 0x0F0) AC = true;
			else AC = false;
			operand.c[1] = (char)((A >> 4) & 0x0F);
			if ((operand.c[1] > 9) || (CY))
			{
				operand.c[1] -= 6;
				if (operand.c[1] & 0x0F0) CY = true;
				else CY = false;
				A -= 0x60;
				A = (char)(A & 0x00FF);
			}
			Z = (A == 0);
			S = (A & 0x0080 != 0);
			PV = Parity(A);
		}
		break;
	case 0340:	//	rpo
		if (!PV)
		{
			PC = m_MemMap.Read2(SP);
			SP += 2;
		}
		break;
	case 0342:	//	jpo		++
		operand.w = Fetch2();
		if (!PV)
		{
			PC = operand.w;
		}
		break;
	case 0344:	//	cpo		++
		operand.w = (PWORD) (PC + 2);
		if (!PV)
		{
			m_MemMap.Write(--SP, operand.c[1]);
			m_MemMap.Write(--SP, operand.c[0]);
			operand.w = Fetch2();
		}
		PC = operand.w;
		break;
	case 0350:	//	rpe
		if (PV)
		{
			PC = m_MemMap.Read2(SP);
			SP += 2;
		}
		break;
	case 0352:	//	jpe		++
		operand.w = Fetch2();
		if (PV)
		{
			PC = operand.w;
		}
		break;
	case 0354:	//	cpe		++
		operand.w = (PWORD) (PC + 2);
		if (PV)
		{
			m_MemMap.Write(--SP, operand.c[1]);
			m_MemMap.Write(--SP, operand.c[0]);
			operand.w = Fetch2();
		}
		PC = operand.w;
		break;
	case 0361:	//	pop		psw
		operand.w = m_MemMap.Read2(SP);
		SP += 2;
		S = ((operand.c[0] & 0x080) != 0);
		Z = ((operand.c[0] & 0x040) != 0);
		Z5 = ((operand.c[0] & 0x020) != 0);	//	undocumented behavior
		AC = ((operand.c[0] & 0x010) != 0);
		Z3 = ((operand.c[0] & 0x008) != 0);	//	undocumented behavior
		PV = ((operand.c[0] & 0x004) != 0);
		AS = ((operand.c[0] & 0x002) != 0);
		CY = ((operand.c[0] & 0x001) != 0);
		A = operand.c[1];
		break;
	case 0365:	//	push psw
		m_MemMap.Write(--SP, A);
		operand.c[0] = (PBYTE) (
			(S ? 0x080 : 0) +
			(Z ? 0x040 : 0) +
			(Z5 ? 0x020: 0) +	//	State of PSW[5] is not documented -- bit 5 of result.
			(AC ? 0x010 : 0) +
			(Z3 ? 0x008 : 0) +	//	State of PSW[3] is not documented -- bit 3 of result.
			(PV ? 0x004 : 0) +
			(AS ? 0x002 : 0) +
			(CY ? 0x001 : 0));
		m_MemMap.Write(--SP, operand.c[0]);
		break;

//	These opcodes are unique to the particular processor:

	case 0010:	//	EXAF on Z-80
		operand.c[0] = A;
		A = A1;
		A1 = operand.c[0];
		bFlag = CY1;
		CY1 = CY;
		CY = bFlag;
		bFlag = PV1;
		PV1 = PV;
		PV = bFlag;
		bFlag = AS1;
		AS1 = AS;
		AS = bFlag;
		bFlag = Z31;
		Z31 = Z3;
		Z3 = bFlag;
		bFlag = AC1;
		AC1 = AC;
		AC = bFlag;
		bFlag = Z51;
		Z51 = Z5;
		Z5 = bFlag;
		bFlag = Z1;
		Z1 = Z;
		Z = bFlag;
		bFlag = S1;
		S1 = S;
		S = bFlag;
		break;
	case 0020:	//	DJNZ rel_disp on Z-80
		++PC;
		nStatus = UNIMPLEMENTED;
		break;
	case 0030:	//	JR rel_disp on Z-80
		xop.c[0] = Fetch();
		PC = (PWORD) (PC + (PDISP) xop.c[0]);
		break;
	case 0040:	//	JRNZ rel_disp on Z-80
		xop.c[0] = Fetch();
		if (!Z)
		{
			PC = (PWORD) (PC + (PDISP) xop.c[0]);
		}
		break;
	case 0050:	//	JRZ rel_disp on Z-80
		xop.c[0] = Fetch();
		if (Z)
		{
			PC = (PWORD) (PC + (PDISP) xop.c[0]);
		}
		break;
	case 0060:	//	JRNC rel_disp on Z-80
		xop.c[0] = Fetch();
		if (!CY)
		{
			PC = (PWORD) (PC + (PDISP) xop.c[0]);
		}
		break;
	case 0070:	//	JRC rel_disp on Z-80
		xop.c[0] = Fetch();
		if (CY)
		{
			PC = (PWORD) (PC + (PDISP) xop.c[0]);
		}
		break;
	case 0313:	//	(CB) lead-in for a group of 256 two-byte opcodes on Z-80
		nOP = Fetch();
		switch(nOP)	//	all values of nOP are valid: rotate and bit-set operations
		{
		case 0x000:
		case 0x001:
		case 0x002:
		case 0x003:
		case 0x004:
		case 0x005:
		case 0x006:
		case 0x007:
		case 0x008:
		case 0x009:
		case 0x00A:
		case 0x00B:
		case 0x00C:
		case 0x00D:
		case 0x00E:
		case 0x00F:
		case 0x010:
		case 0x011:
		case 0x012:
		case 0x013:
		case 0x014:
		case 0x015:
		case 0x016:
		case 0x017:
		case 0x018:
		case 0x019:
		case 0x01A:
		case 0x01B:
		case 0x01C:
		case 0x01D:
		case 0x01E:
		case 0x01F:
		case 0x020:
		case 0x021:
		case 0x022:
		case 0x023:
		case 0x024:
		case 0x025:
		case 0x026:
		case 0x027:
		case 0x028:
		case 0x029:
		case 0x02A:
		case 0x02B:
		case 0x02C:
		case 0x02D:
		case 0x02E:
		case 0x02F:
		case 0x030:
		case 0x031:
		case 0x032:
		case 0x033:
		case 0x034:
		case 0x035:
		case 0x036:
		case 0x037:
		case 0x038:
		case 0x039:
		case 0x03A:
		case 0x03B:
		case 0x03C:
		case 0x03D:
		case 0x03E:
		case 0x03F:
			nStatus = UNIMPLEMENTED;
			break;

//	Bit test group -- state of S & PV flags affected but undocumented.
//	In the emulator, S & PV will be unchanged (like CY).

		case 0100:
		case 0101:
		case 0102:
		case 0103:
		case 0104:
		case 0105:
		case 0106:
		case 0107:		//	test bit zero
			switch(nOP & 7)
			{
			case 0:
				Z = ((B & 1) == 0);
				break;
			case 1:
				Z = ((C & 1) == 0);
				break;
			case 2:
				Z = ((D & 1) == 0);
				break;
			case 3:
				Z = ((E & 1) == 0);
				break;
			case 4:
				Z = ((H & 1) == 0);
				break;
			case 5:
				Z = ((L & 1) == 0);
				break;
			case 6:
				Z = ((m_MemMap.Read(HL) & 1) == 0);
				break;
			case 7:
				Z = ((A & 1) == 0);
				break;
			}
			AC = true;
			AS = false;
			break;
		case 0110:
		case 0111:
		case 0112:
		case 0113:
		case 0114:
		case 0115:
		case 0116:
		case 0117:		//	test bit one
			switch(nOP & 7)
			{
			case 0:
				Z = ((B & 2) == 0);
				break;
			case 1:
				Z = ((C & 2) == 0);
				break;
			case 2:
				Z = ((D & 2) == 0);
				break;
			case 3:
				Z = ((E & 2) == 0);
				break;
			case 4:
				Z = ((H & 2) == 0);
				break;
			case 5:
				Z = ((L & 2) == 0);
				break;
			case 6:
				Z = ((m_MemMap.Read(HL) & 2) == 0);
				break;
			case 7:
				Z = ((A & 2) == 0);
				break;
			}
			AC = true;
			AS = false;
			break;
		case 0120:
		case 0121:
		case 0122:
		case 0123:
		case 0124:
		case 0125:
		case 0126:
		case 0127:		//	test bit two
			switch(nOP & 7)
			{
			case 0:
				Z = ((B & 4) == 0);
				break;
			case 1:
				Z = ((C & 4) == 0);
				break;
			case 2:
				Z = ((D & 4) == 0);
				break;
			case 3:
				Z = ((E & 4) == 0);
				break;
			case 4:
				Z = ((H & 4) == 0);
				break;
			case 5:
				Z = ((L & 4) == 0);
				break;
			case 6:
				Z = ((m_MemMap.Read(HL) & 4) == 0);
				break;
			case 7:
				Z = ((A & 4) == 0);
				break;
			}
			AC = true;
			AS = false;
			break;
		case 0130:
		case 0131:
		case 0132:
		case 0133:
		case 0134:
		case 0135:
		case 0136:
		case 0137:		//	test bit three
			switch(nOP & 7)
			{
			case 0:
				Z = ((B & 8) == 0);
				break;
			case 1:
				Z = ((C & 8) == 0);
				break;
			case 2:
				Z = ((D & 8) == 0);
				break;
			case 3:
				Z = ((E & 8) == 0);
				break;
			case 4:
				Z = ((H & 8) == 0);
				break;
			case 5:
				Z = ((L & 8) == 0);
				break;
			case 6:
				Z = ((m_MemMap.Read(HL) & 8) == 0);
				break;
			case 7:
				Z = ((A & 8) == 0);
				break;
			}
			AC = true;
			AS = false;
			break;
		case 0140:
		case 0141:
		case 0142:
		case 0143:
		case 0144:
		case 0145:
		case 0146:
		case 0147:		//	test bit four
			switch(nOP & 7)
			{
			case 0:
				Z = ((B & 16) == 0);
				break;
			case 1:
				Z = ((C & 16) == 0);
				break;
			case 2:
				Z = ((D & 16) == 0);
				break;
			case 3:
				Z = ((E & 16) == 0);
				break;
			case 4:
				Z = ((H & 16) == 0);
				break;
			case 5:
				Z = ((L & 16) == 0);
				break;
			case 6:
				Z = ((m_MemMap.Read(HL) & 16) == 0);
				break;
			case 7:
				Z = ((A & 16) == 0);
				break;
			}
			AC = true;
			AS = false;
			break;
		case 0150:
		case 0151:
		case 0152:
		case 0153:
		case 0154:
		case 0155:
		case 0156:
		case 0157:		//	test bit five
			switch(nOP & 7)
			{
			case 0:
				Z = ((B & 32) == 0);
				break;
			case 1:
				Z = ((C & 32) == 0);
				break;
			case 2:
				Z = ((D & 32) == 0);
				break;
			case 3:
				Z = ((E & 32) == 0);
				break;
			case 4:
				Z = ((H & 32) == 0);
				break;
			case 5:
				Z = ((L & 32) == 0);
				break;
			case 6:
				Z = ((m_MemMap.Read(HL) & 32) == 0);
				break;
			case 7:
				Z = ((A & 32) == 0);
				break;
			}
			AC = true;
			AS = false;
			break;
		case 0160:
		case 0161:
		case 0162:
		case 0163:
		case 0164:
		case 0165:
		case 0166:
		case 0167:		//	test bit six
			switch(nOP & 7)
			{
			case 0:
				Z = ((B & 64) == 0);
				break;
			case 1:
				Z = ((C & 64) == 0);
				break;
			case 2:
				Z = ((D & 64) == 0);
				break;
			case 3:
				Z = ((E & 64) == 0);
				break;
			case 4:
				Z = ((H & 64) == 0);
				break;
			case 5:
				Z = ((L & 64) == 0);
				break;
			case 6:
				Z = ((m_MemMap.Read(HL) & 64) == 0);
				break;
			case 7:
				Z = ((A & 64) == 0);
				break;
			}
			AC = true;
			AS = false;
			break;
		case 0170:
		case 0171:
		case 0172:
		case 0173:
		case 0174:
		case 0175:
		case 0176:
		case 0177:		//	test bit seven
			switch(nOP & 7)
			{
			case 0:
				Z = ((B & 0x080) == 0);
				break;
			case 1:
				Z = ((C & 0x080) == 0);
				break;
			case 2:
				Z = ((D & 0x080) == 0);
				break;
			case 3:
				Z = ((E & 0x080) == 0);
				break;
			case 4:
				Z = ((H & 0x080) == 0);
				break;
			case 5:
				Z = ((L & 0x080) == 0);
				break;
			case 6:
				Z = ((m_MemMap.Read(HL) & 0x080) == 0);
				break;
			case 7:
				Z = ((A & 0x080) == 0);
				break;
			}
			AC = true;
			AS = false;
			break;
		case 0200:
		case 0201:
		case 0202:
		case 0203:
		case 0204:
		case 0205:
		case 0206:
		case 0207:	//	reset bit zero group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B &= ~1;
				break;
			case 1:	//	c
				C &= ~1;
				break;
			case 2:	//	d
				D &= ~1;
				break;
			case 3:	//	e
				E &= ~1;
				break;
			case 4:	//	h
				H &= ~1;
				break;
			case 5:	//	l
				L &= ~1;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData &= ~1;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A &= ~1;
				break;
			}
			break;
		case 0210:
		case 0211:
		case 0212:
		case 0213:
		case 0214:
		case 0215:
		case 0216:
		case 0217:	//	reset bit one group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B &= ~2;
				break;
			case 1:	//	c
				C &= ~2;
				break;
			case 2:	//	d
				D &= ~2;
				break;
			case 3:	//	e
				E &= ~2;
				break;
			case 4:	//	h
				H &= ~2;
				break;
			case 5:	//	l
				L &= ~2;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData &= ~2;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A &= ~2;
				break;
			}
			break;
		case 0220:
		case 0221:
		case 0222:
		case 0223:
		case 0224:
		case 0225:
		case 0226:
		case 0227:	//	reset bit two group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B &= ~4;
				break;
			case 1:	//	c
				C &= ~4;
				break;
			case 2:	//	d
				D &= ~4;
				break;
			case 3:	//	e
				E &= ~4;
				break;
			case 4:	//	h
				H &= ~4;
				break;
			case 5:	//	l
				L &= ~4;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData &= ~4;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A &= ~4;
				break;
			}
			break;
		case 0230:
		case 0231:
		case 0232:
		case 0233:
		case 0234:
		case 0235:
		case 0236:
		case 0237:	//	reset bit three group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B &= ~8;
				break;
			case 1:	//	c
				C &= ~8;
				break;
			case 2:	//	d
				D &= ~8;
				break;
			case 3:	//	e
				E &= ~8;
				break;
			case 4:	//	h
				H &= ~8;
				break;
			case 5:	//	l
				L &= ~8;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData &= ~8;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A &= ~8;
				break;
			}
			break;
		case 0240:
		case 0241:
		case 0242:
		case 0243:
		case 0244:
		case 0245:
		case 0246:
		case 0247:	//	reset bit four group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B &= ~16;
				break;
			case 1:	//	c
				C &= ~16;
				break;
			case 2:	//	d
				D &= ~16;
				break;
			case 3:	//	e
				E &= ~16;
				break;
			case 4:	//	h
				H &= ~16;
				break;
			case 5:	//	l
				L &= ~16;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData &= ~16;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A &= ~16;
				break;
			}
			break;
		case 0250:
		case 0251:
		case 0252:
		case 0253:
		case 0254:
		case 0255:
		case 0256:
		case 0257:	//	reset bit five group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B &= ~32;
				break;
			case 1:	//	c
				C &= ~32;
				break;
			case 2:	//	d
				D &= ~32;
				break;
			case 3:	//	e
				E &= ~32;
				break;
			case 4:	//	h
				H &= ~32;
				break;
			case 5:	//	l
				L &= ~32;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData &= ~32;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A &= ~32;
				break;
			}
			break;
		case 0260:
		case 0261:
		case 0262:
		case 0263:
		case 0264:
		case 0265:
		case 0266:
		case 0267:	//	reset bit six group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B &= ~64;
				break;
			case 1:	//	c
				C &= ~64;
				break;
			case 2:	//	d
				D &= ~64;
				break;
			case 3:	//	e
				E &= ~64;
				break;
			case 4:	//	h
				H &= ~64;
				break;
			case 5:	//	l
				L &= ~64;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData &= ~64;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A &= ~64;
				break;
			}
			break;
		case 0270:
		case 0271:
		case 0272:
		case 0273:
		case 0274:
		case 0275:
		case 0276:
		case 0277:	//	reset bit seven group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B &= ~0x080;
				break;
			case 1:	//	c
				C &= ~0x080;
				break;
			case 2:	//	d
				D &= ~0x080;
				break;
			case 3:	//	e
				E &= ~0x080;
				break;
			case 4:	//	h
				H &= ~0x080;
				break;
			case 5:	//	l
				L &= ~0x080;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData &= ~0x080;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A &= ~0x080;
				break;
			}
			break;
		case 0300:
		case 0301:
		case 0302:
		case 0303:
		case 0304:
		case 0305:
		case 0306:
		case 0307:	//	set bit zero group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B |= 1;
				break;
			case 1:	//	c
				C |= 1;
				break;
			case 2:	//	d
				D |= 1;
				break;
			case 3:	//	e
				E |= 1;
				break;
			case 4:	//	h
				H |= 1;
				break;
			case 5:	//	l
				L |= 1;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData |= 1;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A |= 1;
				break;
			}
			break;
		case 0310:
		case 0311:
		case 0312:
		case 0313:
		case 0314:
		case 0315:
		case 0316:
		case 0317:	//	set bit one group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B |= 0x080;
				break;
			case 1:	//	c
				C |= 0x080;
				break;
			case 2:	//	d
				D |= 0x080;
				break;
			case 3:	//	e
				E |= 0x080;
				break;
			case 4:	//	h
				H |= 0x080;
				break;
			case 5:	//	l
				L |= 0x080;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData |= 0x080;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A |= 0x080;
				break;
			}
			break;
		case 0320:
		case 0321:
		case 0322:
		case 0323:
		case 0324:
		case 0325:
		case 0326:
		case 0327:	//	set bit two group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B |= 4;
				break;
			case 1:	//	c
				C |= 4;
				break;
			case 2:	//	d
				D |= 4;
				break;
			case 3:	//	e
				E |= 4;
				break;
			case 4:	//	h
				H |= 4;
				break;
			case 5:	//	l
				L |= 4;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData |= 4;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A |= 4;
				break;
			}
			break;
		case 0330:
		case 0331:
		case 0332:
		case 0333:
		case 0334:
		case 0335:
		case 0336:
		case 0337:	//	set bit three group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B |= 8;
				break;
			case 1:	//	c
				C |= 8;
				break;
			case 2:	//	d
				D |= 8;
				break;
			case 3:	//	e
				E |= 8;
				break;
			case 4:	//	h
				H |= 8;
				break;
			case 5:	//	l
				L |= 8;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData |= 8;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A |= 8;
				break;
			}
			break;
		case 0340:
		case 0341:
		case 0342:
		case 0343:
		case 0344:
		case 0345:
		case 0346:
		case 0347:	//	set bit four group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B |= 16;
				break;
			case 1:	//	c
				C |= 16;
				break;
			case 2:	//	d
				D |= 16;
				break;
			case 3:	//	e
				E |= 16;
				break;
			case 4:	//	h
				H |= 16;
				break;
			case 5:	//	l
				L |= 16;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData |= 16;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A |= 16;
				break;
			}
			break;
		case 0350:
		case 0351:
		case 0352:
		case 0353:
		case 0354:
		case 0355:
		case 0356:
		case 0357:	//	set bit five group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B |= 32;
				break;
			case 1:	//	c
				C |= 32;
				break;
			case 2:	//	d
				D |= 32;
				break;
			case 3:	//	e
				E |= 32;
				break;
			case 4:	//	h
				H |= 32;
				break;
			case 5:	//	l
				L |= 32;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData |= 32;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A |= 32;
				break;
			}
			break;
		case 0360:
		case 0361:
		case 0362:
		case 0363:
		case 0364:
		case 0365:
		case 0366:
		case 0367:	//	set bit six group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B |= 64;
				break;
			case 1:	//	c
				C |= 64;
				break;
			case 2:	//	d
				D |= 64;
				break;
			case 3:	//	e
				E |= 64;
				break;
			case 4:	//	h
				H |= 64;
				break;
			case 5:	//	l
				L |= 64;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData |= 64;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A |= 64;
				break;
			}
			break;
		case 0370:
		case 0371:
		case 0372:
		case 0373:
		case 0374:
		case 0375:
		case 0376:
		case 0377:	//	set bit seven group
			switch(nOP & 0x07)
			{
			case 0:	//	b
				B |= 0x080;
				break;
			case 1:	//	c
				C |= 0x080;
				break;
			case 2:	//	d
				D |= 0x080;
				break;
			case 3:	//	e
				E |= 0x080;
				break;
			case 4:	//	h
				H |= 0x080;
				break;
			case 5:	//	l
				L |= 0x080;
				break;
			case 6:	//	m
				nData = m_MemMap.Read(HL);
				nData |= 0x080;
				m_MemMap.Write(HL, nData);
			case 7:	//	a
				A |= 0x080;
				break;
			}
			break;
		default:
			nStatus =	ILLEGAL_OP;	//	CAN'T HAPPEN
			break;
		}
		break;
	case 0331:	//	EXX on Z-80

//	Interchange BC <-> BC', DE <-> DE', HL <-> HL'

		operand.w = BC;
		BC = BC1;
		BC1 = operand.w;
		operand.w = DE;
		DE = DE1;
		DE1 = operand.w;
		operand.w = HL;
		HL = HL1;
		HL1 = operand.w;
		break;
	case 0335:	//	(DD) lead-in for a group of extended opcodes on Z-80: most undocumented, useless duplicate of "bare" opcode
		nOP = (char) Fetch();
		switch(nOP)
		{
		case 0x024:
		case 0x025:
		case 0x026:
		case 0x02C:
		case 0x02D:
		case 0x02E:
		case 0x044:
		case 0x045:
		case 0x04C:
		case 0x04D:
		case 0x054:
		case 0x055:
		case 0x05C:
		case 0x05D:
		case 0x060:
		case 0x061:
		case 0x062:
		case 0x063:
		case 0x064:
		case 0x065:
		case 0x067:
		case 0x068:
		case 0x069:
		case 0x06A:
		case 0x06B:
		case 0x06C:
		case 0x06D:
		case 0x06F:
		case 0x07C:
		case 0x07D:
		case 0x084:
		case 0x085:
		case 0x08C:
		case 0x08D:
		case 0x094:
		case 0x095:
		case 0x09C:
		case 0x09D:
		case 0x0A4:
		case 0x0A5:
		case 0x0AC:
		case 0x0AD:
		case 0x0B4:
		case 0x0B5:
		case 0x0BC:
		case 0x0BD:
			nStatus = ILLEGAL_OP;
			break;

//	Documented opcodes:

		case 0x009:	//	ADD IX,BC
			nStatus = UNIMPLEMENTED;
			break;
		case 0x019:	//	ADD IX,DE
			nStatus = UNIMPLEMENTED;
			break;
		case 0x021:	//	LD	IX,nn
			nStatus = UNIMPLEMENTED;
			break;
		case 0x022:	//	LD	(nn),IX
			nStatus = UNIMPLEMENTED;
			break;
		case 0x023:	//	INC	IX
			nStatus = UNIMPLEMENTED;
			break;
		case 0x029:	//	ADD	IX,IX
			nStatus = UNIMPLEMENTED;
			break;
		case 0x02A:	//	LD	IX,(nn)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x02B:	//	DEC	IX
			nStatus = UNIMPLEMENTED;
			break;
		case 0x034:	//	INC	(IX+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x035:	//	DEC	(IX+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x036:	//	LD	(IX+d),n
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IX + (PDISP) operand.c[0]), Fetch());
			break;
		case 0x039:	//	ADD	IX,SP
			nStatus = UNIMPLEMENTED;
			break;
		case 0x046:	//	LD	B,(IX+d)
			operand.c[0] = Fetch();
			B = m_MemMap.Read((PWORD) (IX + (PDISP) operand.c[0]));
			break;
		case 0x04E:	//	LD	C,(IX+d)
			operand.c[0] = Fetch();
			C = m_MemMap.Read((PWORD) (IX + (PDISP) operand.c[0]));
			break;
		case 0x056:	//	LD	D,(IX+d)
			operand.c[0] = Fetch();
			D = m_MemMap.Read((PWORD) (IX + (PDISP) operand.c[0]));
			break;
		case 0x05E:	//	LD	E,(IX+d)
			operand.c[0] = Fetch();
			E = m_MemMap.Read((PWORD) (IX + (PDISP) operand.c[0]));
			break;
		case 0x066:	//	LD	H,(IX+d)
			operand.c[0] = Fetch();
			H = m_MemMap.Read((PWORD) (IX + (PDISP) operand.c[0]));
			break;
		case 0x06E:	//	LD	L,(IX+d)
			operand.c[0] = Fetch();
			L = m_MemMap.Read((PWORD) (IX + (PDISP) operand.c[0]));
			break;
		case 0x070:	//	LD	(IX+d),B
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IX + (PDISP) operand.c[0]), B);
			break;
		case 0x071:	//	LD	(IX+d),C
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IX + (PDISP) operand.c[0]), C);
			break;
		case 0x072:	//	LD	(IX+d),D
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IX + (PDISP) operand.c[0]), D);
			break;
		case 0x073:	//	LD	(IX+d),E
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IX + (PDISP) operand.c[0]), E);
			break;
		case 0x074:	//	LD	(IX+d),H
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IX + (PDISP) operand.c[0]), H);
			break;
		case 0x075:	//	LD	(IX+d),L
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IX + (PDISP) operand.c[0]), L);
			break;
		case 0x077:	//	LD	(IX+d),A
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IX + (PDISP) operand.c[0]), A);
			break;
		case 0x07E:	//	LD	A,(IX+d)
			operand.c[0] = Fetch();
			A = m_MemMap.Read((PWORD) (IX + (PDISP) operand.c[0]));
			nStatus = UNIMPLEMENTED;
			break;
		case 0x086:	//	ADD	A,(IX+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x08E:	//	ADC	A,(IX+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x096:	//	SUB	(IX+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x09E:	//	SBC	A,(IX+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0A6:	//	AND	(IX+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0AE:	//	XOR	(IX+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0B6:	//	OR	(IX+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0BE:	//	CP	(IX+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0CB:	//	DDCB prefix -- Rotates and bit-set operations involving (IX+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0E1:	//	POP	IX
			IX = m_MemMap.Read2(SP);
			SP += 2;
			break;
		case 0x0E3:	//	EX	(SP),IX
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0E5:	//	PUSH IX
			operand.w = IX;
			m_MemMap.Write(--SP, operand.c[1]);
			m_MemMap.Write(--SP, operand.c[0]);
			break;
		case 0x0E9:	//	JP	(IX)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0F9:	//	LD	SP,IX
			nStatus = UNIMPLEMENTED;
			break;
		case 0x000:
		case 0x001:
		case 0x002:
		case 0x003:
		case 0x004:
		case 0x005:
		case 0x006:
		case 0x007:
		case 0x008:
		case 0x00A:
		case 0x00B:
		case 0x00C:
		case 0x00D:
		case 0x00E:
		case 0x00F:
		case 0x010:
		case 0x011:
		case 0x012:
		case 0x013:
		case 0x014:
		case 0x015:
		case 0x016:
		case 0x017:
		case 0x018:
		case 0x01A:
		case 0x01B:
		case 0x01C:
		case 0x01D:
		case 0x01E:
		case 0x01F:
		case 0x020:
		case 0x027:
		case 0x028:
		case 0x02F:
		case 0x030:
		case 0x031:
		case 0x032:
		case 0x033:
		case 0x037:
		case 0x038:
		case 0x03A:
		case 0x03B:
		case 0x03C:
		case 0x03D:
		case 0x03E:
		case 0x03F:
		case 0x040:
		case 0x041:
		case 0x042:
		case 0x043:
		case 0x047:
		case 0x048:
		case 0x049:
		case 0x04A:
		case 0x04B:
		case 0x04F:
		case 0x050:
		case 0x051:
		case 0x052:
		case 0x053:
		case 0x057:
		case 0x058:
		case 0x059:
		case 0x05A:
		case 0x05B:
		case 0x05F:
		case 0x076:
		case 0x078:
		case 0x079:
		case 0x07A:
		case 0x07B:
		case 0x07F:
		case 0x080:
		case 0x081:
		case 0x082:
		case 0x083:
		case 0x087:
		case 0x088:
		case 0x089:
		case 0x08A:
		case 0x08B:
		case 0x08F:
		case 0x090:
		case 0x091:
		case 0x092:
		case 0x093:
		case 0x097:
		case 0x098:
		case 0x099:
		case 0x09A:
		case 0x09B:
		case 0x09F:
		case 0x0A0:
		case 0x0A1:
		case 0x0A2:
		case 0x0A3:
		case 0x0A7:
		case 0x0A8:
		case 0x0A9:
		case 0x0AA:
		case 0x0AB:
		case 0x0AF:
		case 0x0B0:
		case 0x0B1:
		case 0x0B2:
		case 0x0B3:
		case 0x0B7:
		case 0x0B8:
		case 0x0B9:
		case 0x0BA:
		case 0x0BB:
		case 0x0BF:
		case 0x0C0:
		case 0x0C1:
		case 0x0C2:
		case 0x0C3:
		case 0x0C4:
		case 0x0C5:
		case 0x0C6:
		case 0x0C7:
		case 0x0C8:
		case 0x0C9:
		case 0x0CA:
		case 0x0CC:
		case 0x0CD:
		case 0x0CE:
		case 0x0CF:
		case 0x0D0:
		case 0x0D1:
		case 0x0D2:
		case 0x0D3:
		case 0x0D4:
		case 0x0D5:
		case 0x0D6:
		case 0x0D7:
		case 0x0D8:
		case 0x0D9:
		case 0x0DA:
		case 0x0DB:
		case 0x0DC:
		case 0x0DD:
		case 0x0DE:
		case 0x0DF:
		case 0x0E0:
		case 0x0E2:
		case 0x0E4:
		case 0x0E6:
		case 0x0E7:
		case 0x0E8:
		case 0x0EA:
		case 0x0EB:
		case 0x0EC:
		case 0x0ED:
		case 0x0EE:
		case 0x0EF:
		case 0x0F0:
		case 0x0F1:
		case 0x0F2:
		case 0x0F3:
		case 0x0F4:
		case 0x0F5:
		case 0x0F6:
		case 0x0F7:
		case 0x0F8:
		case 0x0FA:
		case 0x0FB:
		case 0x0FC:
		case 0x0FD:
		case 0x0FE:
		case 0x0FF:
			
//	Duplicated opcodes -- undo the fetch and continue.

			--PC;
			return Step();
			break;
		default:
			nStatus =	ILLEGAL_OP;	//	CAN'T HAPPEN -- ALL BYTE VALUES ARE ACCOUNTED FOR
			break;
		}
		break;
	case 0355:	//	(ED) lead-in for a group of extended opcodes on Z-80
		nOP = Fetch();
		switch(nOP)
		{

//	Undocumented NOP instructions:

		case 0x000:
		case 0x001:
		case 0x002:
		case 0x003:
		case 0x004:
		case 0x005:
		case 0x006:
		case 0x007:
		case 0x008:
		case 0x009:
		case 0x00A:
		case 0x00B:
		case 0x00C:
		case 0x00D:
		case 0x00E:
		case 0x00F:
		case 0x010:
		case 0x011:
		case 0x012:
		case 0x013:
		case 0x014:
		case 0x015:
		case 0x016:
		case 0x017:
		case 0x018:
		case 0x019:
		case 0x01A:
		case 0x01B:
		case 0x01C:
		case 0x01D:
		case 0x01E:
		case 0x01F:
		case 0x020:
		case 0x021:
		case 0x022:
		case 0x023:
		case 0x024:
		case 0x025:
		case 0x026:
		case 0x027:
		case 0x028:
		case 0x029:
		case 0x02A:
		case 0x02B:
		case 0x02C:
		case 0x02D:
		case 0x02E:
		case 0x02F:
		case 0x030:
		case 0x031:
		case 0x032:
		case 0x033:
		case 0x034:
		case 0x035:
		case 0x036:
		case 0x037:
		case 0x038:
		case 0x039:
		case 0x03A:
		case 0x03B:
		case 0x03C:
		case 0x03D:
		case 0x03E:
		case 0x03F:
		case 0x077:
		case 0x07F:
		case 0x080:
		case 0x081:
		case 0x082:
		case 0x083:
		case 0x084:
		case 0x085:
		case 0x086:
		case 0x087:
		case 0x088:
		case 0x089:
		case 0x08A:
		case 0x08B:
		case 0x08C:
		case 0x08D:
		case 0x08E:
		case 0x08F:
		case 0x090:
		case 0x091:
		case 0x092:
		case 0x093:
		case 0x094:
		case 0x095:
		case 0x096:
		case 0x097:
		case 0x098:
		case 0x099:
		case 0x09A:
		case 0x09B:
		case 0x09C:
		case 0x09D:
		case 0x09E:
		case 0x09F:
		case 0x0A4:
		case 0x0A5:
		case 0x0A6:
		case 0x0A7:
		case 0x0AC:
		case 0x0AD:
		case 0x0AE:
		case 0x0AF:
		case 0x0B4:
		case 0x0B5:
		case 0x0B6:
		case 0x0B7:
		case 0x0BC:
		case 0x0BD:
		case 0x0BE:
		case 0x0BF:
		case 0x0C0:
		case 0x0C1:
		case 0x0C2:
		case 0x0C3:
		case 0x0C4:
		case 0x0C5:
		case 0x0C6:
		case 0x0C7:
		case 0x0C8:
		case 0x0C9:
		case 0x0CA:
		case 0x0CB:
		case 0x0CC:
		case 0x0CD:
		case 0x0CE:
		case 0x0CF:
		case 0x0D0:
		case 0x0D1:
		case 0x0D2:
		case 0x0D3:
		case 0x0D4:
		case 0x0D5:
		case 0x0D6:
		case 0x0D7:
		case 0x0D8:
		case 0x0D9:
		case 0x0DA:
		case 0x0DB:
		case 0x0DC:
		case 0x0DD:
		case 0x0DE:
		case 0x0DF:
		case 0x0E0:
		case 0x0E1:
		case 0x0E2:
		case 0x0E3:
		case 0x0E4:
		case 0x0E5:
		case 0x0E6:
		case 0x0E7:
		case 0x0E8:
		case 0x0E9:
		case 0x0EA:
		case 0x0EB:
		case 0x0EC:
		case 0x0ED:
		case 0x0EE:
		case 0x0EF:
		case 0x0F0:
		case 0x0F1:
		case 0x0F2:
		case 0x0F3:
		case 0x0F4:
		case 0x0F5:
		case 0x0F6:
		case 0x0F7:
		case 0x0F8:
		case 0x0F9:
		case 0x0FA:
		case 0x0FB:
		case 0x0FC:
		case 0x0FD:
		case 0x0FE:
		case 0x0FF:
			break;

//	Undocumented instructions -- illegal for now.

		case 0x04C:
		case 0x04E:
		case 0x054:
		case 0x055:
		case 0x05C:
		case 0x05D:
		case 0x064:
		case 0x065:
		case 0x066:
		case 0x06C:
		case 0x06D:
		case 0x06E:
		case 0x070:
		case 0x071:
		case 0x074:
		case 0x075:
		case 0x076:
		case 0x07C:
		case 0x07D:
		case 0x07E:
			nStatus = ILLEGAL_OP;
			break;

//	Defined instructions:

		case 0x040:	//	IN	B,(C)
			m_nInputTarget = REGNUM_B;
			operand.c[0] = operand.c[1] = C;
			m_nBusAddress = operand.w;
			m_bIORead = true;
			m_bPendingRead = true;
			break;
		case 0x041:	//	OUT	(C),B
			nStatus = UNIMPLEMENTED;
			break;
		case 0x042:	//	SBC	HL,BC
			nStatus = UNIMPLEMENTED;
			break;
		case 0x043:	//	LD	(nn),BC
			operand.w = Fetch2();
			m_MemMap.Write(operand.w++, C);
			m_MemMap.Write(operand.w, B);
			break;
		case 0x044:	//	NEG
			nStatus = UNIMPLEMENTED;
			break;
		case 0x045:	//	RETN
			nStatus = UNIMPLEMENTED;
			break;
		case 0x046:	//	IM	0
			nStatus = UNIMPLEMENTED;
			break;
		case 0x047:	//	LD	I,A
			I = A;
			break;
		case 0x048:	//	IN	C,(C)
			m_nInputTarget = REGNUM_C;
			operand.c[0] = operand.c[1] = C;
			m_nBusAddress = operand.w;
			m_bIORead = true;
			m_bPendingRead = true;
			break;
		case 0x049:	//	OUT	(C),C
			nStatus = UNIMPLEMENTED;
			break;
		case 0x04A:	//	ADC	HL,BC
			nStatus = UNIMPLEMENTED;
			break;
		case 0x04B:	//	LD	BC,(nn)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x04D:	//	RETI
			nStatus = UNIMPLEMENTED;
			break;
		case 0x04F:	//	LD	R,A
			R = A;
			break;
		case 0x050:	//	IN	D,(C)
			m_nInputTarget = REGNUM_D;
			operand.c[0] = operand.c[1] = C;
			m_nBusAddress = operand.w;
			m_bIORead = true;
			m_bPendingRead = true;
			break;
		case 0x051:	//	OUT	(C),D
			nStatus = UNIMPLEMENTED;
			break;
		case 0x052:	//	SBC	HL,DE
			nStatus = UNIMPLEMENTED;
			break;
		case 0x053:	//	LD	(nn),DE
			operand.w = Fetch2();
			m_MemMap.Write(operand.w++, E);
			m_MemMap.Write(operand.w, D);
			break;
		case 0x056:	//	IM	1
			nStatus = UNIMPLEMENTED;
			break;
		case 0x057:	//	LD	A,I
			A = I;
			break;
		case 0x058:	//	IN	E,(C)
			m_nInputTarget = REGNUM_E;
			operand.c[0] = operand.c[1] = C;
			m_nBusAddress = operand.w;
			m_bIORead = true;
			m_bPendingRead = true;
			break;
		case 0x059:	//	OUT	(C),E
			nStatus = UNIMPLEMENTED;
			break;
		case 0x05A:	//	ADC	HL,DE
			nStatus = UNIMPLEMENTED;
			break;
		case 0x05B:	//	LD	DE,(nn)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x05E:	//	IM	2
			nStatus = UNIMPLEMENTED;
			break;
		case 0x05F:	//	LD	A,R
			A = R;
			break;
		case 0x060:	//	IN	H,(C)
			m_nInputTarget = REGNUM_H;
			operand.c[0] = operand.c[1] = C;
			m_nBusAddress = operand.w;
			m_bIORead = true;
			m_bPendingRead = true;
			break;
		case 0x061:	//	OUT	(C),H
			nStatus = UNIMPLEMENTED;
			break;
		case 0x062:	//	SBC	HL,HL
			nStatus = UNIMPLEMENTED;
			break;
		case 0x063:	//	LD	(nn),HL
			operand.w = Fetch2();
			m_MemMap.Write(operand.w++, L);
			m_MemMap.Write(operand.w, H);
			break;
		case 0x067:	//	RRD
			nStatus = UNIMPLEMENTED;
			break;
		case 0x068:	//	IN	L,(C)
			m_nInputTarget = REGNUM_L;
			operand.c[0] = operand.c[1] = C;
			m_nBusAddress = operand.w;
			m_bIORead = true;
			m_bPendingRead = true;
			break;
		case 0x069:	//	OUT	(C),L
			nStatus = UNIMPLEMENTED;
			break;
		case 0x06A:	//	ADC	HL,HL
			nStatus = UNIMPLEMENTED;
			break;
		case 0x06B:	//	LD	HL,(nn)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x06F:	//	RLD
			nStatus = UNIMPLEMENTED;
			break;
		case 0x072:	//	SBC	HL,SP
			nStatus = UNIMPLEMENTED;
			break;
		case 0x073:	//	LD	(nn),SP
			operand.w = Fetch2();
			m_MemMap.Write(operand.w++, (PBYTE)(SP & 0x00FF));
			m_MemMap.Write(operand.w, (PBYTE)((SP >> 8) & 0x00FF));
			break;
		case 0x078:	//	IN	A,(C)
			m_nInputTarget = INPUT_TO_A_WITH_FLAGS;
			operand.c[0] = operand.c[1] = C;
			m_nBusAddress = operand.w;
			m_bIORead = true;
			m_bPendingRead = true;
		case 0x079:	//	OUT	(C),A
			operand.c[0] = operand.c[1] = C;
			m_nBusAddress = operand.w;
			m_nBusWriteData = A;
			m_bIOWrite = true;
			break;
		case 0x07A:	//	ADC	HL,SP
			nStatus = UNIMPLEMENTED;
			break;
		case 0x07B:	//	LD	SP,(nn)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0A0:	//	LDI
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0A1:	//	CPI
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0A2:	//	INI
			m_nInputTarget = INPUT_TO_M_AND_INCREMENT_ONCE;
			operand.c[0] = operand.c[1] = C;
			m_nBusAddress = operand.w;
			m_bIORead = true;
			m_bPendingRead = true;
			break;
		case 0x0A3:	//	OUTI
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0A8:	//	LDD
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0A9:	//	CPD
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0AA:	//	IND
			m_nInputTarget = INPUT_TO_M_AND_DECREMENT_ONCE;
			operand.c[0] = operand.c[1] = C;
			m_nBusAddress = operand.w;
			m_bIORead = true;
			m_bPendingRead = true;
			break;
		case 0x0AB:	//	OUTD
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0B0:	//	LDIR
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0B1:	//	CPIR
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0B2:	//	INIR
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0B3:	//	OTIR
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0B8:	//	LDDR
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0B9:	//	CPDR
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0BA:	//	INDR
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0BB:	//	OTDR
			nStatus = UNIMPLEMENTED;
			break;
		default:
			nStatus =	ILLEGAL_OP;	//	CAN'T HAPPEN -- ALL BYTE VALUES ACCOUNTED FOR
			break;
		}
		break;
	case 0375:	//	(FD) lead-in for a group of extended opcodes on Z-80; virtually
				//	identical to the DD opcode set except IY in place of IX.
		nOP = Fetch();
		switch(nOP)
		{
		case 0x024:
		case 0x025:
		case 0x026:
		case 0x02C:
		case 0x02D:
		case 0x02E:
		case 0x044:
		case 0x045:
		case 0x04C:
		case 0x04D:
		case 0x054:
		case 0x055:
		case 0x05C:
		case 0x05D:
		case 0x060:
		case 0x061:
		case 0x062:
		case 0x063:
		case 0x064:
		case 0x065:
		case 0x067:
		case 0x068:
		case 0x069:
		case 0x06A:
		case 0x06B:
		case 0x06C:
		case 0x06D:
		case 0x06F:
		case 0x07C:
		case 0x07D:
		case 0x084:
		case 0x085:
		case 0x08C:
		case 0x08D:
		case 0x094:
		case 0x095:
		case 0x09C:
		case 0x09D:
		case 0x0A4:
		case 0x0A5:
		case 0x0AC:
		case 0x0AD:
		case 0x0B4:
		case 0x0B5:
		case 0x0BC:
		case 0x0BD:
			nStatus = ILLEGAL_OP;
			break;

//	Documented opcodes:

		case 0x009:	//	ADD IY,BC
			nStatus = UNIMPLEMENTED;
			break;
		case 0x019:	//	ADD IY,DE
			nStatus = UNIMPLEMENTED;
			break;
		case 0x021:	//	LD	IY,nn
			nStatus = UNIMPLEMENTED;
			break;
		case 0x022:	//	LD	(nn),IY
			nStatus = UNIMPLEMENTED;
			break;
		case 0x023:	//	INC	IY
			nStatus = UNIMPLEMENTED;
			break;
		case 0x029:	//	ADD	IY,IY
			nStatus = UNIMPLEMENTED;
			break;
		case 0x02A:	//	LD	IY,(nn)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x02B:	//	DEC	IY
			nStatus = UNIMPLEMENTED;
			break;
		case 0x034:	//	INC	(IY+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x035:	//	DEC	(IY+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x036:	//	LD	(IY+d),n
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IY + (PDISP) operand.c[0]), Fetch());
			break;
		case 0x039:	//	ADD	IY,SP
			nStatus = UNIMPLEMENTED;
			break;
		case 0x046:	//	LD	B,(IY+d)
			operand.c[0] = Fetch();
			B = m_MemMap.Read((PWORD) (IY + (PDISP) operand.c[0]));
			break;
		case 0x04E:	//	LD	C,(IY+d)
			operand.c[0] = Fetch();
			C = m_MemMap.Read((PWORD) (IY + (PDISP) operand.c[0]));
			break;
		case 0x056:	//	LD	D,(IY+d)
			operand.c[0] = Fetch();
			D = m_MemMap.Read((PWORD) (IY + (PDISP) operand.c[0]));
			break;
		case 0x05E:	//	LD	E,(IY+d)
			operand.c[0] = Fetch();
			E = m_MemMap.Read((PWORD) (IY + (PDISP) operand.c[0]));
			break;
		case 0x066:	//	LD	H,(IY+d)
			operand.c[0] = Fetch();
			H = m_MemMap.Read((PWORD) (IY + (PDISP) operand.c[0]));
			break;
		case 0x06E:	//	LD	L,(IY+d)
			operand.c[0] = Fetch();
			L = (char) m_MemMap.Read((PWORD) (IY + (PDISP) operand.c[0]));
			break;
		case 0x070:	//	LD	(IY+d),B
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IY + (PDISP) operand.c[0]), B);
			break;
		case 0x071:	//	LD	(IY+d),C
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IY + (PDISP) operand.c[0]), C);
			break;
		case 0x072:	//	LD	(IY+d),D
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IY + (PDISP) operand.c[0]), D);
			break;
		case 0x073:	//	LD	(IY+d),E
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IY + (PDISP) operand.c[0]), E);
			break;
		case 0x074:	//	LD	(IY+d),H
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IY + (PDISP) operand.c[0]), H);
			break;
		case 0x075:	//	LD	(IY+d),L
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IY + (PDISP) operand.c[0]), L);
			break;
		case 0x077:	//	LD	(IY+d),A
			operand.c[0] = Fetch();
			m_MemMap.Write((PWORD) (IY + (PDISP) operand.c[0]), A);
			break;
		case 0x07E:	//	LD	A,(IY+d)
			operand.c[0] = Fetch();
			A = m_MemMap.Read((PWORD) (IY + (PDISP) operand.c[0]));
			nStatus = UNIMPLEMENTED;
			break;
		case 0x086:	//	ADD	A,(IY+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x08E:	//	ADC	A,(IY+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x096:	//	SUB	(IY+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x09E:	//	SBC	A,(IY+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0A6:	//	AND	(IY+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0AE:	//	XOR	(IY+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0B6:	//	OR	(IY+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0BE:	//	CP	(IY+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0CB:	//	FDCB prefix -- Rotates and bit-set operations involving (IY+d)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0E1:	//	POP	IY
			IY = m_MemMap.Read2(SP);
			SP += 2;
			break;
		case 0x0E3:	//	EX	(SP),IY
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0E5:	//	PUSH IY
			operand.w = IY;
			m_MemMap.Write(--SP, operand.c[1]);
			m_MemMap.Write(--SP, operand.c[0]);
			break;
		case 0x0E9:	//	JP	(IY)
			nStatus = UNIMPLEMENTED;
			break;
		case 0x0F9:	//	LD	SP,IY
			nStatus = UNIMPLEMENTED;
			break;
		case 0x000:
		case 0x001:
		case 0x002:
		case 0x003:
		case 0x004:
		case 0x005:
		case 0x006:
		case 0x007:
		case 0x008:
		case 0x00A:
		case 0x00B:
		case 0x00C:
		case 0x00D:
		case 0x00E:
		case 0x00F:
		case 0x010:
		case 0x011:
		case 0x012:
		case 0x013:
		case 0x014:
		case 0x015:
		case 0x016:
		case 0x017:
		case 0x018:
		case 0x01A:
		case 0x01B:
		case 0x01C:
		case 0x01D:
		case 0x01E:
		case 0x01F:
		case 0x020:
		case 0x027:
		case 0x028:
		case 0x02F:
		case 0x030:
		case 0x031:
		case 0x032:
		case 0x033:
		case 0x037:
		case 0x038:
		case 0x03A:
		case 0x03B:
		case 0x03C:
		case 0x03D:
		case 0x03E:
		case 0x03F:
		case 0x040:
		case 0x041:
		case 0x042:
		case 0x043:
		case 0x047:
		case 0x048:
		case 0x049:
		case 0x04A:
		case 0x04B:
		case 0x04F:
		case 0x050:
		case 0x051:
		case 0x052:
		case 0x053:
		case 0x057:
		case 0x058:
		case 0x059:
		case 0x05A:
		case 0x05B:
		case 0x05F:
		case 0x076:
		case 0x078:
		case 0x079:
		case 0x07A:
		case 0x07B:
		case 0x07F:
		case 0x080:
		case 0x081:
		case 0x082:
		case 0x083:
		case 0x087:
		case 0x088:
		case 0x089:
		case 0x08A:
		case 0x08B:
		case 0x08F:
		case 0x090:
		case 0x091:
		case 0x092:
		case 0x093:
		case 0x097:
		case 0x098:
		case 0x099:
		case 0x09A:
		case 0x09B:
		case 0x09F:
		case 0x0A0:
		case 0x0A1:
		case 0x0A2:
		case 0x0A3:
		case 0x0A7:
		case 0x0A8:
		case 0x0A9:
		case 0x0AA:
		case 0x0AB:
		case 0x0AF:
		case 0x0B0:
		case 0x0B1:
		case 0x0B2:
		case 0x0B3:
		case 0x0B7:
		case 0x0B8:
		case 0x0B9:
		case 0x0BA:
		case 0x0BB:
		case 0x0BF:
		case 0x0C0:
		case 0x0C1:
		case 0x0C2:
		case 0x0C3:
		case 0x0C4:
		case 0x0C5:
		case 0x0C6:
		case 0x0C7:
		case 0x0C8:
		case 0x0C9:
		case 0x0CA:
		case 0x0CC:
		case 0x0CD:
		case 0x0CE:
		case 0x0CF:
		case 0x0D0:
		case 0x0D1:
		case 0x0D2:
		case 0x0D3:
		case 0x0D4:
		case 0x0D5:
		case 0x0D6:
		case 0x0D7:
		case 0x0D8:
		case 0x0D9:
		case 0x0DA:
		case 0x0DB:
		case 0x0DC:
		case 0x0DD:
		case 0x0DE:
		case 0x0DF:
		case 0x0E0:
		case 0x0E2:
		case 0x0E4:
		case 0x0E6:
		case 0x0E7:
		case 0x0E8:
		case 0x0EA:
		case 0x0EB:
		case 0x0EC:
		case 0x0ED:
		case 0x0EE:
		case 0x0EF:
		case 0x0F0:
		case 0x0F1:
		case 0x0F2:
		case 0x0F3:
		case 0x0F4:
		case 0x0F5:
		case 0x0F6:
		case 0x0F7:
		case 0x0F8:
		case 0x0FA:
		case 0x0FB:
		case 0x0FC:
		case 0x0FD:
		case 0x0FE:
		case 0x0FF:
			
//	Duplicated opcodes -- undo the fetch and continue.

			--PC;
			return Step();
			break;
		default:
			nStatus =	ILLEGAL_OP;	//	CAN'T HAPPEN -- ALL BYTE VALUES ARE ACCOUNTED FOR
			break;
		}
		break;
	default:
		nStatus =	ILLEGAL_OP;	//	all varient ops not already caught are illegal.
		break;
	}
	return nStatus;
}

CString	Z80 :: DisassembleVarientOp(PBYTE nOp, short int nW, short int nX)
{
	CString	strCode;
	union
	{
		char		c[2];	// c[0] is low-order, c[1] is high-order
		short int	w;
	} operand, xop;

//	NOTE: will not deal with redundant or conflicting prefix codes.

	operand.w = nW;
	xop.w = nX;
	switch(nOp & 0x00ff)
	{
	case 0010:	//	EXAF on Z-80
		strCode = "EXAF";
		break;
	case 0020:	//	DJNZ rel_disp on Z-80
		strCode.Format("DJNZ  %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0030:	//	JR rel_disp on Z-80
		strCode.Format("JR    %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0040:	//	JRNZ rel_disp on Z-80
		strCode.Format("JRNZ  %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0050:	//	JRZ rel_disp on Z-80
		strCode.Format("JRZ   %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0060:	//	JRNC rel_disp on Z-80
		strCode.Format("JRNC  %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0070:	//	JRC rel_disp on Z-80
		strCode.Format("JRC  %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0313:	//	(CB) lead-in for a group of 256 two-byte opcodes on Z-80
		nOp = operand.c[0];
		operand.c[0] = operand.c[1];
		switch(nOp)	//	all values of nOP are valid
		{
		case 0x000:
		case 0x001:
		case 0x002:
		case 0x003:
		case 0x004:
		case 0x005:
		case 0x006:
		case 0x007:
		case 0x008:
		case 0x009:
		case 0x00A:
		case 0x00B:
		case 0x00C:
		case 0x00D:
		case 0x00E:
		case 0x00F:
		case 0x010:
		case 0x011:
		case 0x012:
		case 0x013:
		case 0x014:
		case 0x015:
		case 0x016:
		case 0x017:
		case 0x018:
		case 0x019:
		case 0x01A:
		case 0x01B:
		case 0x01C:
		case 0x01D:
		case 0x01E:
		case 0x01F:
		case 0x020:
		case 0x021:
		case 0x022:
		case 0x023:
		case 0x024:
		case 0x025:
		case 0x026:
		case 0x027:
		case 0x028:
		case 0x029:
		case 0x02A:
		case 0x02B:
		case 0x02C:
		case 0x02D:
		case 0x02E:
		case 0x02F:
		case 0x030:
		case 0x031:
		case 0x032:
		case 0x033:
		case 0x034:
		case 0x035:
		case 0x036:
		case 0x037:
		case 0x038:
		case 0x039:
		case 0x03A:
		case 0x03B:
		case 0x03C:
		case 0x03D:
		case 0x03E:
		case 0x03F:
		case 0x040:
		case 0x041:
		case 0x042:
		case 0x043:
		case 0x044:
		case 0x045:
		case 0x046:
		case 0x047:
		case 0x048:
		case 0x049:
		case 0x04A:
		case 0x04B:
		case 0x04C:
		case 0x04D:
		case 0x04E:
		case 0x04F:
		case 0x050:
		case 0x051:
		case 0x052:
		case 0x053:
		case 0x054:
		case 0x055:
		case 0x056:
		case 0x057:
		case 0x058:
		case 0x059:
		case 0x05A:
		case 0x05B:
		case 0x05C:
		case 0x05D:
		case 0x05E:
		case 0x05F:
		case 0x060:
		case 0x061:
		case 0x062:
		case 0x063:
		case 0x064:
		case 0x065:
		case 0x066:
		case 0x067:
		case 0x068:
		case 0x069:
		case 0x06A:
		case 0x06B:
		case 0x06C:
		case 0x06D:
		case 0x06E:
		case 0x06F:
		case 0x070:
		case 0x071:
		case 0x072:
		case 0x073:
		case 0x074:
		case 0x075:
		case 0x076:
		case 0x077:
		case 0x078:
		case 0x079:
		case 0x07A:
		case 0x07B:
		case 0x07C:
		case 0x07D:
		case 0x07E:
		case 0x07F:
		case 0x080:
		case 0x081:
		case 0x082:
		case 0x083:
		case 0x084:
		case 0x085:
		case 0x086:
		case 0x087:
		case 0x088:
		case 0x089:
		case 0x08A:
		case 0x08B:
		case 0x08C:
		case 0x08D:
		case 0x08E:
		case 0x08F:
		case 0x090:
		case 0x091:
		case 0x092:
		case 0x093:
		case 0x094:
		case 0x095:
		case 0x096:
		case 0x097:
		case 0x098:
		case 0x099:
		case 0x09A:
		case 0x09B:
		case 0x09C:
		case 0x09D:
		case 0x09E:
		case 0x09F:
		case 0x0A0:
		case 0x0A1:
		case 0x0A2:
		case 0x0A3:
		case 0x0A4:
		case 0x0A5:
		case 0x0A6:
		case 0x0A7:
		case 0x0A8:
		case 0x0A9:
		case 0x0AA:
		case 0x0AB:
		case 0x0AC:
		case 0x0AD:
		case 0x0AE:
		case 0x0AF:
		case 0x0B0:
		case 0x0B1:
		case 0x0B2:
		case 0x0B3:
		case 0x0B4:
		case 0x0B5:
		case 0x0B6:
		case 0x0B7:
		case 0x0B8:
		case 0x0B9:
		case 0x0BA:
		case 0x0BB:
		case 0x0BC:
		case 0x0BD:
		case 0x0BE:
		case 0x0BF:
		case 0x0C0:
		case 0x0C1:
		case 0x0C2:
		case 0x0C3:
		case 0x0C4:
		case 0x0C5:
		case 0x0C6:
		case 0x0C7:
		case 0x0C8:
		case 0x0C9:
		case 0x0CA:
		case 0x0CB:
		case 0x0CC:
		case 0x0CD:
		case 0x0CE:
		case 0x0CF:
		case 0x0D0:
		case 0x0D1:
		case 0x0D2:
		case 0x0D3:
		case 0x0D4:
		case 0x0D5:
		case 0x0D6:
		case 0x0D7:
		case 0x0D8:
		case 0x0D9:
		case 0x0DA:
		case 0x0DB:
		case 0x0DC:
		case 0x0DD:
		case 0x0DE:
		case 0x0DF:
		case 0x0E0:
		case 0x0E1:
		case 0x0E2:
		case 0x0E3:
		case 0x0E4:
		case 0x0E5:
		case 0x0E6:
		case 0x0E7:
		case 0x0E8:
		case 0x0E9:
		case 0x0EA:
		case 0x0EB:
		case 0x0EC:
		case 0x0ED:
		case 0x0EE:
		case 0x0EF:
		case 0x0F0:
		case 0x0F1:
		case 0x0F2:
		case 0x0F3:
		case 0x0F4:
		case 0x0F5:
		case 0x0F6:
		case 0x0F7:
		case 0x0F8:
		case 0x0F9:
		case 0x0FA:
		case 0x0FB:
		case 0x0FC:
		case 0x0FD:
		case 0x0FE:
		case 0x0FF:
			strCode.Format("313 %s [UNIMPLEMENTED]", (LPCSTR) Octal(nOp));
			break;
		default:
			strCode.Format("313 %s [ILLEGAL]", (LPCSTR) Octal(nOp));
			break;
		}
		break;
	case 0331:	//	EXX on Z-80
		strCode = "EXX";
		break;
	case 0335:	//	(DD) lead-in for a group of extended opcodes on Z-80
		nOp = operand.c[0];
		operand.c[0] = operand.c[1];
		switch(nOp)
		{
		case 0x000:
		case 0x001:
		case 0x002:
		case 0x003:
		case 0x004:
		case 0x005:
		case 0x006:
		case 0x007:
		case 0x008:
		case 0x009:
		case 0x00A:
		case 0x00B:
		case 0x00C:
		case 0x00D:
		case 0x00E:
		case 0x00F:
		case 0x010:
		case 0x011:
		case 0x012:
		case 0x013:
		case 0x014:
		case 0x015:
		case 0x016:
		case 0x017:
		case 0x018:
		case 0x019:
		case 0x01A:
		case 0x01B:
		case 0x01C:
		case 0x01D:
		case 0x01E:
		case 0x01F:
		case 0x020:
		case 0x021:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x022:					//	STORE IX to address
			strCode.Format("%s %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]), (LPCSTR) Octal(operand.c[1]));
			break;
		case 0x023:
		case 0x024:
		case 0x025:
		case 0x026:
		case 0x027:
		case 0x028:
		case 0x029:
		case 0x02A:
		case 0x02B:
		case 0x02C:
		case 0x02D:
		case 0x02E:
		case 0x02F:
		case 0x030:
		case 0x031:
		case 0x032:
		case 0x033:
		case 0x034:
		case 0x035:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x036:		//	(IX + d) <- n
			strCode.Format("LD    (IX+%s),%s",
							(LPCSTR) Octal(operand.c[0]),
							(LPCSTR) Octal(operand.c[1]));
			break;
		case 0x037:
		case 0x038:
		case 0x039:
		case 0x03A:
		case 0x03B:
		case 0x03C:
		case 0x03D:
		case 0x03E:
		case 0x03F:
		case 0x040:
		case 0x041:
		case 0x042:
		case 0x043:
		case 0x044:
		case 0x045:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x046:						//	B <- (IX + d)
			strCode.Format("LD    B,(IX+%s)",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x047:
		case 0x048:
		case 0x049:
		case 0x04A:
		case 0x04B:
		case 0x04C:
		case 0x04D:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x04E:							//	C <- (IX + d)
			strCode.Format("LD    C,(IX+%s)",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x04F:
		case 0x050:
		case 0x051:
		case 0x052:
		case 0x053:
		case 0x054:
		case 0x055:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x056:							//	D <- (IX + d)
			strCode.Format("LD    D,(IX+%s)",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x057:
		case 0x058:
		case 0x059:
		case 0x05A:
		case 0x05B:
		case 0x05C:
		case 0x05D:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x05E:							//	E <- (IX + d)
			strCode.Format("LD    E,(IX+%s)",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x05F:
		case 0x060:
		case 0x061:
		case 0x062:
		case 0x063:
		case 0x064:
		case 0x065:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x066:							//	H <- (IX + d)
			strCode.Format("LD    H,(IX+%s)",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x067:
		case 0x068:
		case 0x069:
		case 0x06A:
		case 0x06B:
		case 0x06C:
		case 0x06D:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x06E:							//	L <- (IX + d)
			strCode.Format("LD    L,(IX+%s)",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x06F:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x070:							//	(IX + d) <- B
			strCode.Format("LD    (IX+%s),B",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x071:							//	(IX + d) <- C
			strCode.Format("LD    (IX+%s),C",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x072:							//	(IX + d) <- D
			strCode.Format("LD    (IX+%s),D",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x073:							//	(IX + d) <- E
			strCode.Format("LD    (IX+%s),E",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x074:							//	(IX + d) <- H
			strCode.Format("LD    (IX+%s),H",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x075:							//	(IX + d) <- L
			strCode.Format("LD    (IX+%s),L",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x076:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x077:							//	(IX + d) <- A
			strCode.Format("LD    (IX+%s),A",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x078:
		case 0x079:
		case 0x07A:
		case 0x07B:
		case 0x07C:
		case 0x07D:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x07E:							//	A <- (IX + d)
			strCode.Format("LD    A,(IX+%s)",
							(LPCSTR) Octal(operand.c[0]));
			break;
		case 0x07F:
		case 0x080:
		case 0x081:
		case 0x082:
		case 0x083:
		case 0x084:
		case 0x085:
		case 0x086:
		case 0x087:
		case 0x088:
		case 0x089:
		case 0x08A:
		case 0x08B:
		case 0x08C:
		case 0x08D:
		case 0x08E:
		case 0x08F:
		case 0x090:
		case 0x091:
		case 0x092:
		case 0x093:
		case 0x094:
		case 0x095:
		case 0x096:
		case 0x097:
		case 0x098:
		case 0x099:
		case 0x09A:
		case 0x09B:
		case 0x09C:
		case 0x09D:
		case 0x09E:
		case 0x09F:
		case 0x0A0:
		case 0x0A1:
		case 0x0A2:
		case 0x0A3:
		case 0x0A4:
		case 0x0A5:
		case 0x0A6:
		case 0x0A7:
		case 0x0A8:
		case 0x0A9:
		case 0x0AA:
		case 0x0AB:
		case 0x0AC:
		case 0x0AD:
		case 0x0AE:
		case 0x0AF:
		case 0x0B0:
		case 0x0B1:
		case 0x0B2:
		case 0x0B3:
		case 0x0B4:
		case 0x0B5:
		case 0x0B6:
		case 0x0B7:
		case 0x0B8:
		case 0x0B9:
		case 0x0BA:
		case 0x0BB:
		case 0x0BC:
		case 0x0BD:
		case 0x0BE:
		case 0x0BF:
		case 0x0C0:
		case 0x0C1:
		case 0x0C2:
		case 0x0C3:
		case 0x0C4:
		case 0x0C5:
		case 0x0C6:
		case 0x0C7:
		case 0x0C8:
		case 0x0C9:
		case 0x0CA:
		case 0x0CB:
		case 0x0CC:
		case 0x0CD:
		case 0x0CE:
		case 0x0CF:
		case 0x0D0:
		case 0x0D1:
		case 0x0D2:
		case 0x0D3:
		case 0x0D4:
		case 0x0D5:
		case 0x0D6:
		case 0x0D7:
		case 0x0D8:
		case 0x0D9:
		case 0x0DA:
		case 0x0DB:
		case 0x0DC:
		case 0x0DD:
		case 0x0DE:
		case 0x0DF:
		case 0x0E0:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x0E1:
			strCode = "POP   IX";
			break;
		case 0x0E2:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x0E3:	//	(SP) <-> IX
			strCode = "XTIX";
			break;
		case 0x0E4:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x0E5:
			strCode = "PUSH  IX";
			break;
		case 0x0E6:
		case 0x0E7:
		case 0x0E8:
		case 0x0E9:
		case 0x0EA:
		case 0x0EB:
		case 0x0EC:
		case 0x0ED:
		case 0x0EE:
		case 0x0EF:
		case 0x0F0:
		case 0x0F1:
		case 0x0F2:
		case 0x0F3:
		case 0x0F4:
		case 0x0F5:
		case 0x0F6:
		case 0x0F7:
		case 0x0F8:
		case 0x0F9:
		case 0x0FA:
		case 0x0FB:
		case 0x0FC:
		case 0x0FD:
		case 0x0FE:
		case 0x0FF:
			strCode.Format("335 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		default:
			strCode.Format("335 %s %s ILLEGAL", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		}
		break;
	case 0355:	//	(ED) lead-in for a group of extended opcodes on Z-80
		nOp = operand.c[0];
		operand.c[0] = operand.c[1];
		switch(nOp)
		{
		case 0x000:
		case 0x001:
		case 0x002:
		case 0x003:
		case 0x004:
		case 0x005:
		case 0x006:
		case 0x007:
		case 0x008:
		case 0x009:
		case 0x00A:
		case 0x00B:
		case 0x00C:
		case 0x00D:
		case 0x00E:
		case 0x00F:
		case 0x010:
		case 0x011:
		case 0x012:
		case 0x013:
		case 0x014:
		case 0x015:
		case 0x016:
		case 0x017:
		case 0x018:
		case 0x019:
		case 0x01A:
		case 0x01B:
		case 0x01C:
		case 0x01D:
		case 0x01E:
		case 0x01F:
		case 0x020:
		case 0x021:
		case 0x022:
		case 0x023:
		case 0x024:
		case 0x025:
		case 0x026:
		case 0x027:
		case 0x028:
		case 0x029:
		case 0x02A:
		case 0x02B:
		case 0x02C:
		case 0x02D:
		case 0x02E:
		case 0x02F:
		case 0x030:
		case 0x031:
		case 0x032:
		case 0x033:
		case 0x034:
		case 0x035:
		case 0x036:
		case 0x037:
		case 0x038:
		case 0x039:
		case 0x03A:
		case 0x03B:
		case 0x03C:
		case 0x03D:
		case 0x03E:
		case 0x03F:
		case 0x040:
		case 0x041:
		case 0x042:
		case 0x043:
		case 0x044:
		case 0x045:
		case 0x046:
			strCode.Format("355 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x047:
			strCode = "LDIA";
			break;
		case 0x048:
		case 0x049:
		case 0x04A:
		case 0x04B:
		case 0x04C:
		case 0x04D:
		case 0x04E:
			strCode.Format("355 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x04F:
			strCode = "LDRA";
			break;
		case 0x050:
		case 0x051:
		case 0x052:
		case 0x053:
		case 0x054:
		case 0x055:
		case 0x056:
			strCode.Format("355 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x057:
			strCode = "LDAI";
			break;
		case 0x058:
		case 0x059:
		case 0x05A:
		case 0x05B:
		case 0x05C:
		case 0x05D:
		case 0x05E:
			strCode.Format("355 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x05F:
			strCode = "LDAR";
			break;
		case 0x060:
		case 0x061:
		case 0x062:
		case 0x063:
		case 0x064:
		case 0x065:
		case 0x066:
		case 0x067:
		case 0x068:
		case 0x069:
		case 0x06A:
		case 0x06B:
		case 0x06C:
		case 0x06D:
		case 0x06E:
		case 0x06F:
		case 0x070:
		case 0x071:
		case 0x072:
		case 0x073:
		case 0x074:
		case 0x075:
		case 0x076:
		case 0x077:
			strCode.Format("355 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x078:
			strCode = "INAC";
		case 0x079:
			strCode = "OUTC";
			break;
		case 0x07A:
		case 0x07B:
		case 0x07C:
		case 0x07D:
		case 0x07E:
		case 0x07F:
		case 0x080:
		case 0x081:
		case 0x082:
		case 0x083:
		case 0x084:
		case 0x085:
		case 0x086:
		case 0x087:
		case 0x088:
		case 0x089:
		case 0x08A:
		case 0x08B:
		case 0x08C:
		case 0x08D:
		case 0x08E:
		case 0x08F:
		case 0x090:
		case 0x091:
		case 0x092:
		case 0x093:
		case 0x094:
		case 0x095:
		case 0x096:
		case 0x097:
		case 0x098:
		case 0x099:
		case 0x09A:
		case 0x09B:
		case 0x09C:
		case 0x09D:
		case 0x09E:
		case 0x09F:
		case 0x0A0:
		case 0x0A1:
		case 0x0A2:
		case 0x0A3:
		case 0x0A4:
		case 0x0A5:
		case 0x0A6:
		case 0x0A7:
		case 0x0A8:
		case 0x0A9:
		case 0x0AA:
		case 0x0AB:
		case 0x0AC:
		case 0x0AD:
		case 0x0AE:
		case 0x0AF:
		case 0x0B0:
		case 0x0B1:
		case 0x0B2:
		case 0x0B3:
		case 0x0B4:
		case 0x0B5:
		case 0x0B6:
		case 0x0B7:
		case 0x0B8:
		case 0x0B9:
		case 0x0BA:
		case 0x0BB:
		case 0x0BC:
		case 0x0BD:
		case 0x0BE:
		case 0x0BF:
		case 0x0C0:
		case 0x0C1:
		case 0x0C2:
		case 0x0C3:
		case 0x0C4:
		case 0x0C5:
		case 0x0C6:
		case 0x0C7:
		case 0x0C8:
		case 0x0C9:
		case 0x0CA:
		case 0x0CB:
		case 0x0CC:
		case 0x0CD:
		case 0x0CE:
		case 0x0CF:
		case 0x0D0:
		case 0x0D1:
		case 0x0D2:
		case 0x0D3:
		case 0x0D4:
		case 0x0D5:
		case 0x0D6:
		case 0x0D7:
		case 0x0D8:
		case 0x0D9:
		case 0x0DA:
		case 0x0DB:
		case 0x0DC:
		case 0x0DD:
		case 0x0DE:
		case 0x0DF:
		case 0x0E0:
		case 0x0E1:
		case 0x0E2:
		case 0x0E3:
		case 0x0E4:
		case 0x0E5:
		case 0x0E6:
		case 0x0E7:
		case 0x0E8:
		case 0x0E9:
		case 0x0EA:
		case 0x0EB:
		case 0x0EC:
		case 0x0ED:
		case 0x0EE:
		case 0x0EF:
		case 0x0F0:
		case 0x0F1:
		case 0x0F2:
		case 0x0F3:
		case 0x0F4:
		case 0x0F5:
		case 0x0F6:
		case 0x0F7:
		case 0x0F8:
		case 0x0F9:
		case 0x0FA:
		case 0x0FB:
		case 0x0FC:
		case 0x0FD:
		case 0x0FE:
		case 0x0FF:
			strCode.Format("355 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		default:
			strCode.Format("355 %s %s ILLEGAL", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		}
		break;
	case 0375:	//	(FD) lead-in for a group of extended opcodes on Z-80
		nOp = operand.c[0];
		operand.c[0] = operand.c[1];
		switch(nOp)
		{
		case 0x000:
		case 0x001:
		case 0x002:
		case 0x003:
		case 0x004:
		case 0x005:
		case 0x006:
		case 0x007:
		case 0x008:
		case 0x009:
		case 0x00A:
		case 0x00B:
		case 0x00C:
		case 0x00D:
		case 0x00E:
		case 0x00F:
		case 0x010:
		case 0x011:
		case 0x012:
		case 0x013:
		case 0x014:
		case 0x015:
		case 0x016:
		case 0x017:
		case 0x018:
		case 0x019:
		case 0x01A:
		case 0x01B:
		case 0x01C:
		case 0x01D:
		case 0x01E:
		case 0x01F:
		case 0x020:
		case 0x021:
		case 0x022:
		case 0x023:
		case 0x024:
		case 0x025:
		case 0x026:
		case 0x027:
		case 0x028:
		case 0x029:
		case 0x02A:
		case 0x02B:
		case 0x02C:
		case 0x02D:
		case 0x02E:
		case 0x02F:
		case 0x030:
		case 0x031:
		case 0x032:
		case 0x033:
		case 0x034:
		case 0x035:
		case 0x036:
		case 0x037:
		case 0x038:
		case 0x039:
		case 0x03A:
		case 0x03B:
		case 0x03C:
		case 0x03D:
		case 0x03E:
		case 0x03F:
		case 0x040:
		case 0x041:
		case 0x042:
		case 0x043:
		case 0x044:
		case 0x045:
		case 0x046:
		case 0x047:
		case 0x048:
		case 0x049:
		case 0x04A:
		case 0x04B:
		case 0x04C:
		case 0x04D:
		case 0x04E:
		case 0x04F:
		case 0x050:
		case 0x051:
		case 0x052:
		case 0x053:
		case 0x054:
		case 0x055:
		case 0x056:
		case 0x057:
		case 0x058:
		case 0x059:
		case 0x05A:
		case 0x05B:
		case 0x05C:
		case 0x05D:
		case 0x05E:
		case 0x05F:
		case 0x060:
		case 0x061:
		case 0x062:
		case 0x063:
		case 0x064:
		case 0x065:
		case 0x066:
		case 0x067:
		case 0x068:
		case 0x069:
		case 0x06A:
		case 0x06B:
		case 0x06C:
		case 0x06D:
		case 0x06E:
		case 0x06F:
		case 0x070:
		case 0x071:
		case 0x072:
		case 0x073:
		case 0x074:
		case 0x075:
		case 0x076:
		case 0x077:
		case 0x078:
		case 0x079:
		case 0x07A:
		case 0x07B:
		case 0x07C:
		case 0x07D:
		case 0x07E:
		case 0x07F:
		case 0x080:
		case 0x081:
		case 0x082:
		case 0x083:
		case 0x084:
		case 0x085:
		case 0x086:
		case 0x087:
		case 0x088:
		case 0x089:
		case 0x08A:
		case 0x08B:
		case 0x08C:
		case 0x08D:
		case 0x08E:
		case 0x08F:
		case 0x090:
		case 0x091:
		case 0x092:
		case 0x093:
		case 0x094:
		case 0x095:
		case 0x096:
		case 0x097:
		case 0x098:
		case 0x099:
		case 0x09A:
		case 0x09B:
		case 0x09C:
		case 0x09D:
		case 0x09E:
		case 0x09F:
		case 0x0A0:
		case 0x0A1:
		case 0x0A2:
		case 0x0A3:
		case 0x0A4:
		case 0x0A5:
		case 0x0A6:
		case 0x0A7:
		case 0x0A8:
		case 0x0A9:
		case 0x0AA:
		case 0x0AB:
		case 0x0AC:
		case 0x0AD:
		case 0x0AE:
		case 0x0AF:
		case 0x0B0:
		case 0x0B1:
		case 0x0B2:
		case 0x0B3:
		case 0x0B4:
		case 0x0B5:
		case 0x0B6:
		case 0x0B7:
		case 0x0B8:
		case 0x0B9:
		case 0x0BA:
		case 0x0BB:
		case 0x0BC:
		case 0x0BD:
		case 0x0BE:
		case 0x0BF:
		case 0x0C0:
		case 0x0C1:
		case 0x0C2:
		case 0x0C3:
		case 0x0C4:
		case 0x0C5:
		case 0x0C6:
		case 0x0C7:
		case 0x0C8:
		case 0x0C9:
		case 0x0CA:
		case 0x0CB:
		case 0x0CC:
		case 0x0CD:
		case 0x0CE:
		case 0x0CF:
		case 0x0D0:
		case 0x0D1:
		case 0x0D2:
		case 0x0D3:
		case 0x0D4:
		case 0x0D5:
		case 0x0D6:
		case 0x0D7:
		case 0x0D8:
		case 0x0D9:
		case 0x0DA:
		case 0x0DB:
		case 0x0DC:
		case 0x0DD:
		case 0x0DE:
		case 0x0DF:
		case 0x0E0:
			strCode.Format("375 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x0E1:
			strCode = "POP   IY";
			break;
		case 0x0E2:
		case 0x0E3:
		case 0x0E4:
			strCode.Format("375 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		case 0x0E5:
			strCode = "PUSH  IY";
			break;
		case 0x0E6:
		case 0x0E7:
		case 0x0E8:
		case 0x0E9:
		case 0x0EA:
		case 0x0EB:
		case 0x0EC:
		case 0x0ED:
		case 0x0EE:
		case 0x0EF:
		case 0x0F0:
		case 0x0F1:
		case 0x0F2:
		case 0x0F3:
		case 0x0F4:
		case 0x0F5:
		case 0x0F6:
		case 0x0F7:
		case 0x0F8:
		case 0x0F9:
		case 0x0FA:
		case 0x0FB:
		case 0x0FC:
		case 0x0FD:
		case 0x0FE:
		case 0x0FF:
			strCode.Format("375 %s %s UNIMPLEMENTED", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		default:
			strCode.Format("375 %s %s ILLEGAL", (LPCSTR) Octal(nOp), (LPCSTR) Octal(operand.c[0]));
			break;
		}
		break;
	default:
		strCode.Format("%s %s %s [ILLEGAL]",
			(LPCSTR) Octal(nOp),
			(LPCSTR) Octal(operand.c[0]),
			(LPCSTR) Octal(operand.c[1]));
		break;
	}
	return strCode;
}

//	============================================================================
//	Methods of the I8080 emulator:


//	TODO:	Need to implement constructor, destructor and Reset() method.


int		I8080 :: VarientOp(PBYTE nOP)
{
	union
	{
		PBYTE		c[2];	// c[0] is low-order, c[1] is high-order
		PWORD		w;
	} operand;
	int	nStatus = 0;

	switch(nOP)
	{
	case 0047:	//	daa

//	1.	If the least signifiant 4 bits are greater than 9 or if AC is true, add
//		6 to A.
//	2.	Now if the most significant 4 bits are greater than 9 or if CY is true,
//		add 0x60 to A.
//	Affects all flags.

		operand.c[0] = (PBYTE)(A & 0x0F);
		if ((operand.c[0] > 9) || (AC))
		{
			operand.c[0] += 6;
			operand.c[1] = A;
			if ((operand.c[1] + 6) > 0x00FF) CY = true;
			A += 6;
		}
		if (operand.c[0] > 0x0F) AC = true;
		else AC = false;
		operand.c[1] = (PBYTE)((A >> 4) & 0x0F);
		if ((operand.c[1] > 9) || (CY))
		{
			operand.c[1] += 6;
			if (operand.c[1] > 0x0F) CY = true;
			else CY = false;
			A += 0x60;
		}
		Z = (A == 0);
		S = (A & 0x0080 != 0);
		P = Parity(A);
		break;
	case 0340:	//	rpo
		if (!P)
		{
			PC = m_MemMap.Read2(SP);
			SP += 2;
		}
		break;
	case 0342:	//	jpo		++
		operand.w = Fetch2();
		if (!P)
		{
			PC = operand.w;
		}
		break;
	case 0344:	//	cpo		++
		operand.w = (PWORD) (PC + 2);
		if (!P)
		{
			m_MemMap.Write(--SP, operand.c[1]);
			m_MemMap.Write(--SP, operand.c[0]);
			operand.w = Fetch2();
		}
		PC = operand.w;
		break;
	case 0350:	//	rpe
		if (P)
		{
			PC = m_MemMap.Read2(SP);
			SP += 2;
		}
		break;
	case 0352:	//	jpe		++
		operand.w = Fetch2();
		if (P)
		{
			PC = operand.w;
		}
		break;
	case 0354:	//	cpe		++
		operand.w = (PWORD) (PC + 2);
		if (P)
		{
			m_MemMap.Write(--SP, operand.c[1]);
			m_MemMap.Write(--SP, operand.c[0]);
			operand.w = Fetch2();
		}
		PC = operand.w;
		break;
	case 0361:	//	pop		psw
		operand.w = m_MemMap.Read2(SP);
		SP += 2;
		S = ((operand.c[0] & 0x080) != 0);
		Z = ((operand.c[0] & 0x040) != 0);
		//	flags byte bit 5 is not used
		AC = ((operand.c[0] & 0x010) != 0);
		//	flags byte bit 3 is not used
		P = ((operand.c[0] & 0x004) != 0);
		//	flags byte bit 1 is not used
		CY = ((operand.c[0] & 0x001) != 0);
		A = operand.c[1];
		break;
	case 0365:	//	push psw
		m_MemMap.Write(--SP, A);
		operand.c[0] = (PBYTE) (
			(S ? 0x080 : 0) +
			(Z ? 0x040 : 0) +
			0 +					//	Flags[5] was always 0
			(AC ? 0x010 : 0) +
			0 +					//	Flags[3] was always 0
			(P ? 0x004 : 0) +
			0x002 +				//	Flags[1] was always 1
			(CY ? 0x001 : 0));
		m_MemMap.Write(--SP, operand.c[0]);
		break;
	default:
		nStatus = ILLEGAL_OP;	//	all varient ops are illegal on 8080 except PUSH/POP psw.
		break;
	}
	return nStatus;
}


CString	I8080 :: DisassembleVarientOp(PBYTE nOp, short int /* nW */, short int /* nX */)
{
	CString	strCode;

	strCode.Format("%s [ILLEGAL OPCODE]", (LPCSTR) Octal(nOp));
	return strCode;
}

//	============================================================================
//	Methods of the I8085 emulator:

//	----------------------------------------------------------------------------
//
//		Undocumented 8085 Instructions:
//
//		0010	DSUB					0x008
//
//			Double Subtract
//
//				HL <- HL + BC
//
//			Cycles:	3
//			States:	10
//			Flags:	Z, S, P, C, AC, X5, V
//
//		0020	ARHL					0x010
//
//			Arithmetic right shift of HL
//
//				HL <- (HL / 2)
//			Cycles:	2
//			States:	7
//			Flags:	C
//
//		0030	RDEL					0x018
//
//			Rotate DE left through Carry
//
//				Temp <- DE >> 15 & 0x00001
//				DE <- DE << 1 || C
//				C <- Temp
//
//			Cycles:	3
//			States:	10
//			Flags:	C, V
//
//		0050	LDHI	n				0x028
//
//			Load DE from HL plus immediate byte
//
//				DE <- HL + next byte
//
//			Cycles:	3
//			States:	10
//			Flags:	none
//
//		0070	LDSI	n				0x038
//
//			Load DE from SP plus immediate byte
//
//				DE <- SP + next byte
//
//			Cycles:	3
//			States:	10
//			Flags:	none
//
//		0313	RSTV					0x0CB
//
//			Restart (call to location 000.100 [0x00040]) if overflow
//
//			if (V)
//			{
//			  *(--SP) <- (PC >> 8) & 0x000FF
//			  *(--SP) <- (PC & 0x000FF
//			  PC <- 0x00040
//			}
//
//			Cycles:	(V ? 3 : 1)
//			States:	(V ? 12 : 6)
//			Flags:	none
//
//		0331	SHLX					0x0D9
//
//			Store HL indirect through DE
//
//			*(DE) <- L
//			*(DE + 1) <- H
//
//			Cycles:	3
//			States:	10
//			Flags:	none
//
//		0335	JNX5	aa				0x0DD
//
//			if (!X5)
//			{
//			  PC <- immediate address
//			}
//			else
//			{
//			  PC += 2
//			}
//
//			Cycles:	(X5 ? 2 : 3)
//			States:	(X5 ? 10 : 7)
//			Flags:	none
//
//		0355	LHLX					0x0ED
//
//			Load HL indirect through DE
//
//			L <- *(DE)
//			H <- *(DE+1)
//
//			Cycles:	3
//			States:	10
//			Flags:	none
//
//		0375	JX5		aa			0x0FD
//
//			if (X5)
//			{
//			  PC <- immediate address
//			}
//			else
//			{
//			  PC += 2
//			}
//
//			Cycles:	(X5 ? 3 : 2)
//			States:	(X5 ? 7 : 10)
//			Flags:	none
//
//	----------------------------------------------------------------------------



//	TODO:	Need to implement constructor, destructor and Reset() method.

int		I8085 :: VarientOp(PBYTE nOP)
{

//	IMPLEMENTATION BUG: Cannot be used as an INTERRUPT Instruction

	union
	{
		PBYTE		c[2];	// c[0] is low-order, c[1] is high-order
		PWORD		w;
	} operand;
	int		nData;
	int		nStatus = 0;

	switch(nOP)
	{
	case 0047:	//	daa

//	1.	If the least signifiant 4 bits are greater than 9 or if AC is true, add
//		6 to A.
//	2.	Now if the most significant 4 bits are greater than 9 or if CY is true,
//		add 0x60 to A.
//	Affects all flags.

		operand.c[0] = (PBYTE)(A & 0x0F);
		if ((operand.c[0] > 9) || (AC))
		{
			operand.c[0] += 6;
			operand.c[1] = A;
			if ((operand.c[1] + 6) > 0x00FF) CY = true;
			A += 6;
		}
		if (operand.c[0] > 0x0F) AC = true;
		else AC = false;
		operand.c[1] = (PBYTE)((A >> 4) & 0x0F);
		if ((operand.c[1] > 9) || (CY))
		{
			operand.c[1] += 6;
			if (operand.c[1] > 0x0F) CY = true;
			else CY = false;
			A += 0x60;
		}
		Z = (A == 0);
		S = (A & 0x0080 != 0);
		P = Parity(A);
		break;
	case 0010:	//	undocumented DSUB instruction in 8085s
		SubtractRegisterPairs(&HL, &BC);
		break;
	case 0020:	//	undocumented ARHL instruction in 8085s
		nData = HL;
		nData /= 2;
		HL = (PWORD) nData;

//	TODO: affects CY.  (Probably clears it.)

		break;
	case 0030:	//	undocumented RDEL instruction in 8085s
		nData = (DE << 1) | (CY ? 1 : 0);
		CY = ((DE & 0x08000) != 0);
		DE = (PWORD) nData;

//	TODO: affects V.

		break;
	case 0040:	//	RIM instruction in 8085s
		nStatus = UNIMPLEMENTED;
		break;
	case 0050:	//	undocumented LDHI instruction in 8085s
		operand.c[0] = Fetch();
		operand.c[1] = 0;
		DE = (PWORD) (HL + operand.w);
		break;
	case 0060:	//	SIM instruction in 8085s
		nStatus = UNIMPLEMENTED;
		break;
	case 0070:	//	undocumented LDSI instruction in 8085s
		operand.c[0] = Fetch();
		operand.c[1] = 0;
		DE = (PWORD) (SP + operand.w);
		break;
	case 0313:	//	undocumented RSTV instruction in 8085s
		nStatus = UNIMPLEMENTED;
		break;
	case 0331:	//	undocumented SHLX instruction in 8085s
		m_MemMap.Write(DE, L);
		m_MemMap.Write((PWORD)(DE + 1), H);
		break;
	case 0335:	//	undocumented JNX5 instruction in 8085s
		operand.w = Fetch2();
		if (X5)
		{
			PC = operand.w;
		}
		break;
	case 0355:	//	undocumented LHLX instruction in 8085s
		HL = m_MemMap.Read2(DE);
		break;
	case 0340:	//	rpo
		if (!P)
		{
			PC = m_MemMap.Read2(SP);
			SP += 2;
		}
		break;
	case 0342:	//	jpo		++
		operand.w = Fetch2();
		if (!P)
		{
			PC = operand.w;
		}
		break;
	case 0344:	//	cpo		++
		operand.w = (PWORD) (PC + 2);
		if (!P)
		{
			m_MemMap.Write(--SP, operand.c[1]);
			m_MemMap.Write(--SP, operand.c[0]);
			operand.w = Fetch2();
		}
		PC = operand.w;
		break;
	case 0350:	//	rpe
		if (P)
		{
			PC = m_MemMap.Read2(SP);
			SP += 2;
		}
		break;
	case 0352:	//	jpe		++
		operand.w = Fetch2();
		if (P)
		{
			PC = operand.w;
		}
		break;
	case 0354:	//	cpe		++
		operand.w = (PWORD) (PC + 2);
		if (P)
		{
			m_MemMap.Write(--SP, operand.c[1]);
			m_MemMap.Write(--SP, operand.c[0]);
			operand.w = Fetch2();
		}
		PC = operand.w;
		break;
	case 0361:	//	pop		psw
		operand.w = m_MemMap.Read2(SP);
		SP += 2;
		S = ((operand.c[0] & 0x080) != 0);
		Z = ((operand.c[0] & 0x040) != 0);
		X5 = ((operand.c[0] & 0x020) != 0);
		AC = ((operand.c[0] & 0x010) != 0);
											//	flags byte bit 3 is not used
		P = ((operand.c[0] & 0x004) != 0);
		V = ((operand.c[0] & 0x002) != 0);
		CY = ((operand.c[0] & 0x001) != 0);
		A = operand.c[1];
		break;
	case 0365:	//	push psw
		m_MemMap.Write(--SP, A);
		operand.c[0] = (PBYTE) (
			(S ? 0x080 : 0) +
			(Z ? 0x040 : 0) +
			(X5 ? 0x020 : 0) +
			(AC ? 0x010 : 0) +
			0 +					//	Flags[3] was always 0
			(P ? 0x004 : 0) +
			(V ? 0x002 : 0) +
			(CY ? 0x001 : 0));
		m_MemMap.Write(--SP, operand.c[0]);
		break;
	case 0375:	//	undocumented JX5 instruction in 8085s
		operand.w = Fetch2();
		if (!X5)
		{
			PC = operand.w;
		}
		break;
	default:
		nStatus =	ILLEGAL_OP;	//	all varient ops not already caught are illegal.
		break;
	}
	return nStatus;
}

void			Processor :: SubtractRegisterPairs(PWORD * pTarget,
												PWORD * pOperand)
{
	PWORD		nOp1;
	PWORD		nOp2;
	long		nDifference;

//	We go to the trouble of having both the target and the operand register
//	specified because the Z-80 opcode map has extra double-subtract operations.

	nOp1 = *pTarget;
	nOp2 = *pOperand;
	nDifference = (((long) nOp1) & 0x00FFFF) -
					(((long) nOp2) & 0x00FFFF);
	CY = (nDifference < 0);

//	S unaffected
//	Z unaffected
//	P unaffected
//	V unaffected

	*pTarget = (PWORD) nDifference;

//					sign_of_operand_1 && !sign_of_operand_2 ||
//					sign_of_operand_1 && sign_of_result ||
//					!sign_of_operand_2 && sign_of_result

	X5= (((nOp1 & 0x08000) && !(nOp2 & 0x08000)) ||
		((nOp1 & 0x08000) && (*pTarget & 0x08000)) ||
		(!(nOp2 & 0x08000) && (*pTarget & 0x08000)));

//	The 8085, needs to compute V.  Don't know what it should be.

//	PV unaffected.
	AS = false;
}


CString	I8085 :: DisassembleVarientOp(PBYTE nOp, short int nW, short int /* nX */)
{
	CString	strCode;

	union
	{
		char		c[2];	// c[0] is low-order, c[1] is high-order
		short int	w;
	} operand;

	operand.w = nW;
	switch(nOp)
	{
	case 0010:	//	undocumented DSUB instruction in 8085s
		strCode = "DSUB";
		break;
	case 0020:	//	undocumented ARHL instruction in 8085s
		strCode = "ARHL";
		break;
	case 0030:	//	undocumented RDEL instruction in 8085s
		strCode = "RDEL";
		break;
	case 0040:	//	RIM instruction in 8085s
		strCode = "RIM";
		break;
	case 0050:	//	undocumented LDHI instruction in 8085s
		strCode.Format("LDHI  %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0060:	//	SIM instruction in 8085s
		strCode = "SIM";
		break;
	case 0070:	//	undocumented LDSI instruction in 8085s
		strCode.Format("LDSI  %s", (LPCSTR) Octal(operand.c[0]));
		break;
	case 0313:	//	undocumented RSTV instruction in 8085s
		strCode = "RSTV";
		break;
	case 0331:	//	undocumented SHLX instruction in 8085s
		strCode = "SHLX";
		break;
	case 0335:	//	undocumented JNX5 instruction in 8085s
		strCode.Format("JNX5  %s", (LPCSTR) SplitOctal(operand.w));
		break;
	case 0355:	//	undocumented LHLX instruction in 8085s
		strCode = "LHLX";
		break;
	case 0375:	//	undocumented JX5 instruction in 8085s
		strCode.Format("JX5   %s", (LPCSTR) SplitOctal(operand.w));
		break;
	default:
		strCode.Format("%s [ILLEGAL OPCODE]", (LPCSTR) Octal(nOp));
		break;
	}
	return strCode;
}
