/******************************************************************************
 *                                                                            *
 *      HDOS Disk dump utility -- creates Virtual H-8 Disk Image from the     *
 *      data on the disk in SY1: and displays it on TT: (with the intention   *
 *      that the display is being captured by a terminal emulator on a PC.    *
 *                                                                            *
 *      Copyright (c) 2001 by   David A. Wallace                              *
 *                              154 Dunstable Rd.                             *
 *                              N.Chelmsford, MA 01863                        *
 *                                                                            *
 *      Version 1.0 of 06/12/2001 -- To be compiled with Toolworks C/80       *
 *                                                                            *
 ******************************************************************************/

#ifdef  _MSC_VER

/*
 *  Note: This program can be tested by compiling it as a console application
 *  for any Windows 32 platform (NT, 9x, 2000 or ME) using Microsoft Visual C++
 *  version 5.0 or higher.  In test mode, no disk I/O will be performed and
 *  the dump will consist of sectors containing all zeros.  The timing in test
 *  mode tries to account for the diskette seek, latency and transfer time
 *  and for the communication speed, but it does not compensate for the speed
 *  (or rather lack thereof) of the Z80 processor running the code.  In test
 *  mode, the program takes about ten minutes to simulate the disk dump of
 *  one diskette; in actuality, the H-8 executing the code compiled using
 *  C/80 takes 40 minutes.
 */

#include    <stdio.h>
#include    <process.h>
#include    <Windows.h>

#else

#include "printf.c"

#endif


/*
 *  Set #defines for using device SY1: as the diskette device and set the
 *  side, track and sector limits appropriate to single-sided H17 diskettes.
 */

#define DEVICE  1
#define SIDES   1
#define TRACKS  40
#define SECTORS 10


/*
 *  HDOS-isms: SCALL definition and the SCALLs we reference and locations in the
 *  HDOS  parameters area we reference from C code.  (In order of appearance)
 */

#define SCALL           RST 7
#define HDOS_DISMOUNT   DB  201Q

#define AIO_UNI 020461  /*  AIO.UNI:    041.061A -- unit of device          */
#define D_TT    020240  /*  D.TT:       040.240A -- track number            */
#define D_TS    020241  /*  D.TS:       040.241A -- sector number           */


/*
    +-----------------------------------------------------------------------+
    |                                                                       |
    |                                   Subroutines                         |
    |                                                                       |
    +-----------------------------------------------------------------------+
 */


/*
 *  Octal() -- Convert a value between 0 and 255 to a decimal number between
 *  0 and 377 such that the new value is the three octal digits that represents
 *  the old value.  Example: converts a decimal 173 to a value of 255 which is
 *  the octal number whose value is 173, decimal.
 */

int Octal(n)
{
    n = n & 0x00FF;
    return (((n) / 64) * 100 + (((n) % 64) / 8) * 10 + ((n) % 8));
}


/*
 *  SeekAndReadSector() -- Seeks to the specified side, track and sector of the
 *  given device and then reads 256 bytes from that sector into the supplied
 *  256-character buffer.
 */

int SeekAndReadSector(nDev, nSide, nTrack, nSector, szData)
    int nDev, nSide,
/*  nSide is currently ignored. */
         nTrack, nSector;
    char * szData;
{

#ifndef _MSC_VER        /*  Uses code that will fail in MSC under Win32.    */

    char *  pHDOS;

/*
 *  Set the device to SY<nDev>:.
 */

    pHDOS = AIO_UNI;
    *pHDOS = (char) nDev;

/*
 *  Store the track number in D_TT.
 */

    pHDOS = D_TT;
    *pHDOS = (char) nTrack;

/*
 *  The sector is stored in D_TS.
 */

    pHDOS = D_TS;
    *pHDOS = (char) nSector;

/*
 *  Read Sector, returning data in szData.
 */

    pHDOS = &szData[0];
	pHDOS;

#asm
*
*   Physical sector level read operation.  Reads 256 bytes at track/sector
*   specified by D_TT and D_TS from volume in drive specified by AIO_UNI.
*   Does not check volume number while positioning head.
*
*   Warning: Dismount the device first or HDOS will fail!
*
*   (AIO_UNI) = unit number
*   (D_TT)    = track
*   (D_TS)    = sector
*   (HL)      = address of buffer


*	HDOS symbols used by this code:

D_LPSA  EQU     040116A
D_READ  EQU     040147A
D_DTS   EQU     040163A
D_SDT   EQU     040166A
D_LPS   EQU     040177A
D_RDB   EQU     040202A
D_STS   EQU     040210A
D_STZ   EQU     040213A
D_WSC   EQU     040221A
D_ERRT  EQU     040232A
DTT_    EQU     040240A
D_DLYHS EQU     040244A

*****************************************************************************
*																			
*
*	Read Physical Sector Without Regard to Volume Number in Sector Header	
*
*																			
*
*****************************************************************************

READ    PUSH    H
*
*	Patch the jump vectors for D.DTS and D.LPS to skip the track-and-sector
*	decode step (use physical sector numbering instead of logical) and to
*	substitute our own LPS routine.
*
        LHLD    D_DTS+1     Set up to re-vector HDOS
        SHLD    X_DTS1
        LHLD    D_LPS+1
        SHLD    X_LPS1
        LXI     H,D_SDT
        SHLD    D_DTS+1
        LXI     H,OURLPS
        SHLD    D_LPS+1
*
*	Perform read.
*
        EI                  Ensure interrupts on
        POP     D           Get address of data buffer
        LXI     B,256       Request 256 bytes
        CALL    D_READ      Do read operation
*
*	Un-do patches to jump vectors.
*
        LHLD    X_DTS1
        SHLD    D_DTS+1
        LHLD    X_LPS1
        SHLD    D_LPS+1
        JMP     READ_1

*
*	Patch save areas
*
X_LPS1  DW      0
X_DTS1  DW      0


*****************************************************************************
*																			
*
*   Our version of D.LPS -- does not verify volume number while reading the	
*
*	sector header; otherwise, it's identical to what's in HDOS.				
*
*																			
*
*****************************************************************************

LPS0    CALL    D_STS       Call "Skip this Sector"
OURLPS  LDA     D_LPSA      A = # of attempts
        MOV     B,A
        LDA     D_DLYHS     Head settling delay
        ANA     A
        JNZ     LPS0        Wait for head
LPS1    DI					Time-critical stuff follows -- d
isable interrupts
        CALL    D_WSC       Wait for sync
        JC      LPS3        That's an oops -- no sync
        CALL    D_RDB       Read byte (volume number -- ignored)
        LXI     H,DTT_      (M) = track wanted
        CALL    D_RDB       Read track number
        CMP     M
        JNZ     LPS5        Wrong track.
        INX     H           (m) = sector number
        CALL    D_RDB       Read sector number
        CMP     M
        JNZ     LPS2        Wrong sector
*
*   Correct sector found.
*
        MOV     H,D			(H) = checksum
        CALL    D_RDB		Read checksum byte
        CMP     H
        RZ                  Success!
        MVI     L,270Q      Code for checksum error
LPS1_1  MVI     H,40Q       (040.xxx) = error byte address
        CALL    D_ERRT      Count the error
LPS2    CALL    D_STS		go around again.
        DCR     B
        JNZ     LPS1        Retry
        STC                 Too many errors -- give up.
        RET
LPS3    MVI     L,266Q      Code for header sync error
        JMP     LPS1_1
LPS5    EI                  Not on right track -- recalibrate
        CALL    D_STZ       Seek to track zero
        CALL    D_SDT       Seek back to correct track.
        MVI     L,272Q      Code for bad track number
        JMP     LPS1_1

READ_1  DS      0

#endasm

#else

/*
 *  Simulate the time delay which HDOS would incur doing the disk seek and the
 *  read operation.  Very roughly, this is 10 milliseconds for track-to-track
 *  seek time every ten sectors (since we are doing consecutive sectors on
 *  consecutive tracks), plus about another 100 milliseconds to find the
 *  sector (due to rotational latency), plus 100 milliseconds for the heads
 *  to engage, plus 20 milliseconds to get the data.
 */

    Sleep(221);

#endif

    return 0;
}

    
/*
 *  GetVolumeNumber() -- Retrieves the volume number of the disk in the
 *  specified device.  The volume number is in the header record (which is the
 *  contents of track zero, sector nine) at byte offset zero.
 */

int     GetVolumeNumber(nDev)
{
    char    szLabel[256];

    SeekAndReadSector(nDev, 0, 0, 9, szLabel);  
    return szLabel[0];
}


/*
 *  Dismount the media in the specified device, using the DMOUNT SCALL.
 */

int Dismount(nDev)
  int nDev;
{

#ifndef _MSC_VER        /*  Uses code that will fail in MSC under Win32.    */

    static char szDev[5];
    char *      pHDOS;

    szDev[0] = 'S';
    szDev[1] = 'Y';
    szDev[2] = '0' + nDev;
    szDev[3] = ':';
    szDev[4] = 0;

/*
 *  Dismount the device: (SCALL 201Q).
 */

    szDev;
#asm
    SCALL
    HDOS_DISMOUNT
#endasm

#endif

    return 0;
}


/*
    +-----------------------------------------------------------------------+
    |                                                                       |
    |                                Main Function                          |
    |                                                                       |
    +-----------------------------------------------------------------------+
 */

main()
{
    register int    nI;
    register int    nByte;
    register int    nSector;
    register int    nTrack;
    int             nSide;
    int             nProtection;
    static char     szData[256];
    int             nDone;
    int             nVolumeNumber;

    printf("Disk dump routine, version 1.0 of 06/12/2001.\n");
    printf("Dumps the disk in SY%d: for use by Virtual H-8.\n\n\n", DEVICE);
    Dismount(DEVICE);
    do
    {
        do
        {

/*
 *  Prompt for a diskette to dump.
 */

            printf("Insert a disk to dump in SY%d:.  Hit RETURN when ready: ",
                    DEVICE);
            while(getchar() != '\n');

/*
 *  Ask for the style of the diskette's write-protect notch.
 */

            printf("Is the disk Locked[L]/Protected[P]/Writable[W]? ");
            nProtection = -1;
            switch(getchar())
            {
            case 'l':
            case 'L':
                nProtection = 2;
                break;
            case 'p':
            case 'P':
                nProtection = 1;
                break;
            case 'w':
            case 'W':
                nProtection = 0;
                break;
            default:
                while (getchar() != '\n');
                printf("Please answer L, P or W.\n");
                break;
            }
        } while (nProtection == -1);
        while (getchar() != '\n');

/*
 *  Get the diskette's volume number -- needed for the header record.
 */

        nVolumeNumber = GetVolumeNumber(DEVICE);

/*
 *  Mark the start of image data.
 */

        printf("\n\n\n-----------------------------------------\n");
        printf("[cut here]\n\n\n");

/*
 *  Output the header.
 */

        printf("377 300 %03d %03d %03d %03d %03d ",
                Octal(nProtection),
                Octal(nVolumeNumber),
                Octal(SIDES),
                Octal(TRACKS),
                Octal(SECTORS));
        printf("000 000 000 000 000 000 000 000 000\n");
        printf("\n");

/*
 *  Prompt for a description and then eat characters up to the newline.  This
 *  apparently non-functional piece of code actually inserts a one-line comment
 *  into the disk image file being captured by the terminal emulator, saving
 *  us the trouble of editing in the description later (if we can even remember
 *  what disk it actually was we were imaging by the time the dump finishes!).
 */
        printf(";Description: ");
        while(getchar() != '\n');
        printf("\n");

/*
 *  This is the actual dump loop -- the logic is tested only for SIDES == 1.
 */

        for (nTrack = 0; nTrack < TRACKS; ++nTrack)
        {
            for (nSide = 0; nSide < SIDES; ++nSide)
            {
                for (nSector = 0; nSector < SECTORS; ++nSector)
                {

/*
 *  Zero out the data buffer.
 */

                    for (nI = 0; nI < 256; ++nI)
                    {

#ifdef _MSC_VER

/*
 *  In simulation mode, set the data buffer to random values.
 */

                        szData[nI] = rand() & 0x0FF;

#else

                        szData[nI] = 0;

#endif

                    }

/*
 *  Read a sector's worth of data.
 */

                    SeekAndReadSector(DEVICE, nSide, nTrack, nSector, szData);

/*
 *  Create the comment that marks the start of a sector.  This comment is not
 *  needed by the software -- it just allows us to make sense of the file when
 *  looking at it with a text editor.
 */

                    printf(";Data: Side %d, Track %02d, Sector %02d\n",
                            nSide, nTrack, nSector);
 
/*
 *  Dump the sector data as sixteen lines of sixteen bytes.
 */

                    for (nByte = 0; nByte < 256; nByte += 16)
                    {
                        for (nI = 0; nI < 16; ++nI)
                        {
                            printf("%03d ", Octal(szData[nByte + nI]));
                        }
                        printf("\n");

#ifdef _MSC_VER

/*  Simulate the 9600-baud terminal I/O delay   */

                        Sleep(67);

#endif

                    }
                }
            }
        }

/*
 *  Mark the end of the image.
 */

        printf("\n\n[cut here]\n\n\n");
        printf("----------------------------------\nEnd of Dump\n");

/*
 *  Allow for more than one diskette image per session.
 */

        do
        {
            nDone = -1;
            printf("Again(Y/N)? ");
            switch(getchar())
            {
            case 'n':
            case 'N':
                nDone = 1;
                break;
            case 'y':
            case 'Y':
                nDone = 0;
                break;
            default:
                while (getchar() != '\n');
                printf("\nPlease answer 'Y' or 'N'.\n");
                nDone = -1;
                break;
            }
            if (nDone != -1)
            {
                while (getchar() != '\n');
            }
            if (nDone == 1)
            {
                exit(0);
            }
        } while (nDone == -1);
    } while (nDone == 0);
}
